"""
pyrad.prod.process_product
==========================

Functions for obtaining Pyrad products from the datasets

.. autosummary::
    :toctree: generated/

    generate_traj_product

"""

from warnings import warn

from ..io.io_aux import get_save_dir, make_filename
from ..io.timeseries import TimeSeries
from ..graph.plots import plot_pos


def generate_traj_product(traj, prdcfg):
    """
    Generates trajectory products. Accepted product types:
        'TRAJ_MAP': Plots the trajectory on a lat-lon map with the altitude
            color coded
        'TRAJ_PLOT': Plots time series of the trajectory respect to the radar
            elevation, azimuth or range
            User defined parameters:
                'datatype': str
                    The type of parameter: 'EL', 'AZ', or 'RANGE'
        'TRAJ_TEXT': Writes the trajectory information in a csv file

    Parameters
    ----------
    traj : Trajectory object

    prdcfg : dictionary of dictionaries
        product configuration dictionary of dictionaries

    Returns
    -------
    None

    """

    dssavedir = prdcfg['dsname']
    if 'dssavename' in prdcfg:
        dssavedir = prdcfg['dssavename']

    if prdcfg['type'] == 'TRAJ_PLOT':

        timeinfo = traj.time_vector[0]

        savedir = get_save_dir(
            prdcfg['basepath'], prdcfg['procname'], dssavedir,
            prdcfg['prdname'], timeinfo=timeinfo)

        ts = TimeSeries("", traj.time_vector,
                        timeformat="%Y-%m-%d %H:%M:%S.%f")

        if prdcfg['datatype'] == 'EL':
            fname = make_filename(
                'ts', prdcfg['dstype'], 'TRAJ', prdcfg['imgformat'],
                prdcfginfo="EL", timeinfo=timeinfo, timeformat='%Y%m%d%H%M%S',
                runinfo=prdcfg['runinfo'])

            ts.add_dataseries(
                "Elevation", "Elevation", "deg",
                traj.radar_list[0].elevation_vec)
            ts.plot(savedir + fname[0])

        elif prdcfg['datatype'] == 'AZ':
            fname = make_filename(
                'ts', prdcfg['dstype'], 'TRAJ', prdcfg['imgformat'],
                prdcfginfo="AZ", timeinfo=timeinfo, timeformat='%Y%m%d%H%M%S',
                runinfo=prdcfg['runinfo'])

            ts.add_dataseries(
                "Azimuth", "Azimuth", "deg", traj.radar_list[0].azimuth_vec)
            ts.plot(savedir + fname[0])

        elif prdcfg['datatype'] == 'RANGE':
            fname = make_filename(
                'ts', prdcfg['dstype'], 'TRAJ', prdcfg['imgformat'],
                prdcfginfo="RANGE", timeinfo=timeinfo,
                timeformat='%Y%m%d%H%M%S', runinfo=prdcfg['runinfo'])

            ts.add_dataseries(
                "Range", "Range", "m", traj.radar_list[0].range_vec)
            ts.plot(savedir + fname[0])

        else:
            raise Exception("ERROR: Unknown datatype '%s' (dataset: '%s')" %
                            (prdcfg['datatype'], prdcfg['dsname']))

        return None

    if prdcfg['type'] == 'TRAJ_TEXT':

        timeinfo = traj.time_vector[0]

        savedir = get_save_dir(prdcfg['basepath'], prdcfg['procname'],
                               dssavedir, prdcfg['prdname'],
                               timeinfo=timeinfo)

        fname = make_filename('ts', prdcfg['dstype'], 'TRAJ', ['csv'],
                              prdcfginfo=None, timeinfo=timeinfo,
                              timeformat='%Y%m%d%H%M%S',
                              runinfo=prdcfg['runinfo'])

        description = ["Description:",
                       "Time series of a plane trajectory in radar "
                       "coordinates."]

        ts = TimeSeries(description, traj.time_vector,
                        timeformat="%Y-%m-%d %H:%M:%S.%f")
        ts.add_dataseries("Elevation", "Elevation", "deg",
                          traj.radar_list[0].elevation_vec)
        ts.add_dataseries("Azimuth", "Azimuth", "deg",
                          traj.radar_list[0].azimuth_vec)
        ts.add_dataseries("Range", "Range", "m", traj.radar_list[0].range_vec)

        ts.add_dataseries("Absolute Speed", "Absolute Speed", "m/s",
                          traj.radar_list[0].v_abs)
        ts.add_dataseries("Radial Speed", "Radial Speed", "m/s",
                          traj.radar_list[0].v_r)
        ts.add_dataseries("Elevation Speed", "Elevation Speed", "deg/s",
                          traj.radar_list[0].v_el)
        ts.add_dataseries("Azimuth Speed", "Azimuth Speed", "deg/s",
                          traj.radar_list[0].v_az)

        ts.write(savedir + fname[0])

        return None

    if prdcfg['type'] == 'TRAJ_MAP': #Trajectory on a map
        timeinfo = traj.time_vector[0]
        savedir = get_save_dir(
            prdcfg['basepath'], prdcfg['procname'], dssavedir,
            prdcfg['prdname'], timeinfo=timeinfo)

        fname = make_filename(
            'ts', prdcfg['dstype'], 'TRAJ', prdcfg['imgformat'],
            prdcfginfo="MAP", timeinfo=timeinfo, timeformat='%Y%m%d%H%M%S',
            runinfo=prdcfg['runinfo'])

        title = "Trajectory Starting at  %s" % \
                traj.time_vector[0].strftime("%Y-%m-%d")

        fname_list = fname
        for i, fname in enumerate(fname_list):
            fname_list[i] = savedir+fname

        # Get traj
        lat = traj.wgs84_lat_deg
        lon = traj.wgs84_lon_deg
        alt = traj.wgs84_alt_m

        plot_pos(
            lat, lon, alt, fname_list, cb_label='Altitude [m]', titl=title,
            save_fig=True)

        return None

    warn(' Unsupported product type: ' + prdcfg['type'])
    return None



"""
pyrad.io.read_data_sun
======================

Functions for reading data used in sun monitoring

.. autosummary::
    :toctree: generated/

    read_sun_hits_multiple_days
    read_sun_hits
    read_sun_retrieval
    read_solar_flux
"""

import datetime
import csv
from warnings import warn

import numpy as np

from pyart.config import get_fillvalue

from .io_aux import get_save_dir, make_filename


def read_sun_hits_multiple_days(cfg, time_ref, nfiles=1):
    """
    Reads sun hits data from multiple file sources

    Parameters
    ----------
    cfg : dict
        dictionary with configuration data to find out the right file
    time_ref : datetime object
        reference time
    nfiles : int
        number of files to read

    Returns
    -------
    date, ray, nrng, rad_el, rad_az, sun_el, sun_az, ph, ph_std, nph, nvalh,
    pv, pv_std, npv, nvalv, zdr, zdr_std, nzdr, nvalzdr : tupple
        Each parameter is an array containing a time series of information on
        a variable

    """
    timeinfo = time_ref - datetime.timedelta(days=nfiles-1)

    date = list()
    ray = list()
    nrng = list()
    rad_el = list()
    rad_az = list()
    sun_el = list()
    sun_az = list()
    ph = np.ma.array(list())
    ph_std = np.ma.array(list())
    nph = list()
    nvalh = list()
    pv = np.ma.array(list())
    pv_std = np.ma.array(list())
    npv = list()
    nvalv = list()
    zdr = np.ma.array(list())
    zdr_std = np.ma.array(list())
    nzdr = list()
    nvalzdr = list()

    for i in range(nfiles):
        savedir = get_save_dir(
            cfg['basepath'], cfg['procname'], cfg['dsname'],
            cfg['sun_hits_dir'], timeinfo=timeinfo, create_dir=False)

        fname = make_filename(
            'info', cfg['type'], 'detected', ['csv'],
            timeinfo=timeinfo, timeformat='%Y%m%d')

        (date_aux, ray_aux, nrng_aux, rad_el_aux, rad_az_aux, sun_el_aux,
         sun_az_aux, ph_aux, ph_std_aux, nph_aux, nvalh_aux, pv_aux,
         pv_std_aux, npv_aux, nvalv_aux, zdr_aux, zdr_std_aux, nzdr_aux,
         nvalzdr_aux) = read_sun_hits(savedir+fname[0])

        if date_aux is None:
            return (None, None, None, None, None, None, None, None, None,
                    None, None, None, None, None, None, None, None, None,
                    None)

        date += date_aux
        ray += list(ray_aux)
        nrng += list(nrng_aux)
        rad_el += list(rad_el_aux)
        rad_az += list(rad_az_aux)
        sun_el += list(sun_el_aux)
        sun_az += list(sun_az_aux)
        ph = np.ma.append(ph, ph_aux)
        ph_std = np.ma.append(ph_std, ph_std_aux)
        nph += list(nph_aux)
        nvalh += list(nvalh_aux)
        pv = np.ma.append(pv, pv_aux)
        pv_std = np.ma.append(pv_std, pv_std_aux)
        npv += list(npv_aux)
        nvalv += list(nvalv_aux)
        zdr = np.ma.append(zdr, zdr_aux)
        zdr_std = np.ma.append(zdr_std, zdr_std_aux)
        nzdr += list(nzdr_aux)
        nvalzdr += list(nvalzdr_aux)

        timeinfo += datetime.timedelta(days=1)

    ray = np.squeeze(np.array(ray))
    nrng = np.squeeze(np.array(nrng))
    rad_el = np.squeeze(np.array(rad_el))
    rad_az = np.squeeze(np.array(rad_az))
    sun_el = np.squeeze(np.array(sun_el))
    sun_az = np.squeeze(np.array(sun_az))
    nph = np.squeeze(np.array(nph))
    nvalh = np.squeeze(np.array(nvalh))
    npv = np.squeeze(np.array(npv))
    nvalv = np.squeeze(np.array(nvalv))
    nzdr = np.squeeze(np.array(nzdr))
    nvalzdr = np.squeeze(np.array(nvalzdr))

    return (date, ray, nrng, rad_el, rad_az, sun_el, sun_az, ph, ph_std, nph,
            nvalh, pv, pv_std, npv, nvalv, zdr, zdr_std, nzdr, nvalzdr)


def read_sun_hits(fname):
    """
    Reads sun hits data contained in a csv file

    Parameters
    ----------
    fname : str
        path of time series file

    Returns
    -------
    date, ray, nrng, rad_el, rad_az, sun_el, sun_az, ph, ph_std, nph, nvalh,
    pv, pv_std, npv, nvalv, zdr, zdr_std, nzdr, nvalzdr : tupple
        Each parameter is an array containing a time series of information on
        a variable

    """
    try:
        with open(fname, 'r', newline='') as csvfile:
            # first count the lines
            reader = csv.DictReader(
                row for row in csvfile if not row.startswith('#'))
            nrows = sum(1 for row in reader)

            ray = np.empty(nrows, dtype=int)
            nrng = np.empty(nrows, dtype=int)
            rad_el = np.empty(nrows, dtype=float)
            rad_az = np.empty(nrows, dtype=float)
            sun_el = np.empty(nrows, dtype=float)
            sun_az = np.empty(nrows, dtype=float)
            ph = np.ma.empty(nrows, dtype=float)
            ph_std = np.ma.empty(nrows, dtype=float)
            nph = np.empty(nrows, dtype=int)
            nvalh = np.empty(nrows, dtype=int)
            pv = np.ma.empty(nrows, dtype=float)
            pv_std = np.ma.empty(nrows, dtype=float)
            npv = np.empty(nrows, dtype=int)
            nvalv = np.empty(nrows, dtype=int)
            zdr = np.ma.empty(nrows, dtype=float)
            zdr_std = np.ma.empty(nrows, dtype=float)
            nzdr = np.empty(nrows, dtype=int)
            nvalzdr = np.empty(nrows, dtype=int)

            # now read the data
            csvfile.seek(0)
            reader = csv.DictReader(
                row for row in csvfile if not row.startswith('#'))
            date = list()
            for i, row in enumerate(reader):
                date.append(datetime.datetime.strptime(
                    row['time'], '%Y-%m-%d %H:%M:%S.%f'))
                ray[i] = int(row['ray'])
                nrng[i] = int(row['NPrng'])
                rad_el[i] = float(row['rad_el'])
                rad_az[i] = float(row['rad_az'])
                sun_el[i] = float(row['sun_el'])
                sun_az[i] = float(row['sun_az'])
                ph[i] = float(row['dBm_sun_hit'])
                ph_std[i] = float(row['std(dBm_sun_hit)'])
                nph[i] = int(row['NPh'])
                nvalh[i] = int(row['NPhval'])
                pv[i] = float(row['dBmv_sun_hit'])
                pv_std[i] = float(row['std(dBmv_sun_hit)'])
                npv[i] = int(row['NPv'])
                nvalv[i] = int(row['NPvval'])
                zdr[i] = float(row['ZDR_sun_hit'])
                zdr_std[i] = float(row['std(ZDR_sun_hit)'])
                nzdr[i] = int(row['NPzdr'])
                nvalzdr[i] = int(row['NPzdrval'])

            ph = np.ma.masked_values(ph, get_fillvalue())
            ph_std = np.ma.masked_values(ph_std, get_fillvalue())
            pv = np.ma.masked_values(pv, get_fillvalue())
            pv_std = np.ma.masked_values(pv_std, get_fillvalue())
            zdr = np.ma.masked_values(zdr, get_fillvalue())
            zdr_std = np.ma.masked_values(zdr_std, get_fillvalue())

            csvfile.close()

            return (date, ray, nrng, rad_el, rad_az, sun_el, sun_az,
                    ph, ph_std, nph, nvalh, pv, pv_std, npv, nvalv,
                    zdr, zdr_std, nzdr, nvalzdr)

    except EnvironmentError as ee:
        warn(str(ee))
        warn('Unable to read file '+fname)
        return (None, None, None, None, None, None, None, None, None, None,
                None, None, None, None, None, None, None, None, None)


def read_sun_retrieval(fname):
    """
    Reads sun retrieval data contained in a csv file

    Parameters
    ----------
    fname : str
        path of time series file

    Returns
    -------
    first_hit_time, last_hit_time, nhits_h, el_width_h, az_width_h, el_bias_h,
    az_bias_h, dBm_sun_est, std_dBm_sun_est, sf_h,
    nhits_v, el_width_v, az_width_v, el_bias_v, az_bias_v, dBmv_sun_est,
    std_dBmv_sun_est, sf_v,
    nhits_zdr, zdr_sun_est, std_zdr_sun_est,
    sf_ref, ref_time : tupple
        Each parameter is an array containing a time series of information on
        a variable

    """
    try:
        with open(fname, 'r', newline='') as csvfile:
            # first count the lines
            reader = csv.DictReader(
                row for row in csvfile if not row.startswith('#'))
            nrows = sum(1 for row in reader)

            nhits_h = np.empty(nrows, dtype=int)
            el_width_h = np.ma.empty(nrows, dtype=float)
            az_width_h = np.ma.empty(nrows, dtype=float)
            el_bias_h = np.ma.empty(nrows, dtype=float)
            az_bias_h = np.ma.empty(nrows, dtype=float)
            dBm_sun_est = np.ma.empty(nrows, dtype=float)
            std_dBm_sun_est = np.ma.empty(nrows, dtype=float)
            sf_h = np.ma.empty(nrows, dtype=float)

            nhits_v = np.empty(nrows, dtype=int)
            el_width_v = np.ma.empty(nrows, dtype=float)
            az_width_v = np.ma.empty(nrows, dtype=float)
            el_bias_v = np.ma.empty(nrows, dtype=float)
            az_bias_v = np.ma.empty(nrows, dtype=float)
            dBmv_sun_est = np.ma.empty(nrows, dtype=float)
            std_dBmv_sun_est = np.ma.empty(nrows, dtype=float)
            sf_v = np.ma.empty(nrows, dtype=float)

            nhits_zdr = np.empty(nrows, dtype=int)
            zdr_sun_est = np.ma.empty(nrows, dtype=float)
            std_zdr_sun_est = np.ma.empty(nrows, dtype=float)

            sf_ref = np.ma.empty(nrows, dtype=float)

            # now read the data
            csvfile.seek(0)
            reader = csv.DictReader(
                row for row in csvfile if not row.startswith('#'))

            first_hit_time = list()
            last_hit_time = list()
            ref_time = list()
            for i, row in enumerate(reader):
                first_hit_time.append(datetime.datetime.strptime(
                    row['first_hit_time'], '%Y%m%d%H%M%S'))
                last_hit_time.append(datetime.datetime.strptime(
                    row['last_hit_time'], '%Y%m%d%H%M%S'))

                nhits_h[i] = int(row['nhits_h'])
                el_width_h[i] = float(row['el_width_h'])
                az_width_h[i] = float(row['az_width_h'])
                el_bias_h[i] = float(row['el_bias_h'])
                az_bias_h[i] = float(row['az_bias_h'])
                dBm_sun_est[i] = float(row['dBm_sun_est'])
                std_dBm_sun_est[i] = float(row['std(dBm_sun_est)'])
                sf_h[i] = float(row['sf_h'])

                nhits_v[i] = int(row['nhits_v'])
                el_width_v[i] = float(row['el_width_v'])
                az_width_v[i] = float(row['az_width_v'])
                el_bias_v[i] = float(row['el_bias_v'])
                az_bias_v[i] = float(row['az_bias_v'])
                dBmv_sun_est[i] = float(row['dBmv_sun_est'])
                std_dBmv_sun_est[i] = float(row['std(dBmv_sun_est)'])
                sf_v[i] = float(row['sf_v'])

                nhits_zdr[i] = int(row['nhits_zdr'])
                zdr_sun_est[i] = float(row['ZDR_sun_est'])
                std_zdr_sun_est[i] = float(row['std(ZDR_sun_est)'])

                sf_ref[i] = float(row['sf_ref'])
                if row['ref_time'] == 'None':
                    ref_time.append(None)
                else:
                    ref_time.append(datetime.datetime.strptime(
                        row['ref_time'], '%Y%m%d%H%M%S'))

            el_width_h = np.ma.masked_values(el_width_h, get_fillvalue())
            az_width_h = np.ma.masked_values(az_width_h, get_fillvalue())
            el_bias_h = np.ma.masked_values(el_bias_h, get_fillvalue())
            az_bias_h = np.ma.masked_values(az_bias_h, get_fillvalue())
            dBm_sun_est = np.ma.masked_values(dBm_sun_est, get_fillvalue())
            std_dBm_sun_est = np.ma.masked_values(
                std_dBm_sun_est, get_fillvalue())
            sf_h = np.ma.masked_values(sf_h, get_fillvalue())

            el_width_v = np.ma.masked_values(el_width_v, get_fillvalue())
            az_width_v = np.ma.masked_values(az_width_v, get_fillvalue())
            el_bias_v = np.ma.masked_values(el_bias_v, get_fillvalue())
            az_bias_v = np.ma.masked_values(az_bias_v, get_fillvalue())
            dBmv_sun_est = np.ma.masked_values(dBmv_sun_est, get_fillvalue())
            std_dBmv_sun_est = np.ma.masked_values(
                std_dBmv_sun_est, get_fillvalue())
            sf_v = np.ma.masked_values(sf_v, get_fillvalue())

            zdr_sun_est = np.ma.masked_values(zdr_sun_est, get_fillvalue())
            std_zdr_sun_est = np.ma.masked_values(
                std_zdr_sun_est, get_fillvalue())

            sf_ref = np.ma.masked_values(sf_ref, get_fillvalue())

            csvfile.close()

            return (first_hit_time, last_hit_time, nhits_h,
                    el_width_h, az_width_h, el_bias_h, az_bias_h,
                    dBm_sun_est, std_dBm_sun_est, sf_h,
                    nhits_v, el_width_v, az_width_v, el_bias_v, az_bias_v,
                    dBmv_sun_est, std_dBmv_sun_est, sf_v,
                    nhits_zdr, zdr_sun_est, std_zdr_sun_est, sf_ref, ref_time)

    except EnvironmentError as ee:
        warn(str(ee))
        warn('Unable to read file '+fname)
        return (None, None, None, None, None, None, None, None, None, None,
                None, None, None, None, None, None, None, None, None, None,
                None, None, None)


def read_solar_flux(fname):
    """
    Reads solar flux data from the DRAO observatory in Canada

    Parameters
    ----------
    fname : str
        path of time series file

    Returns
    -------
    flux_datetime : datetime array
        the date and time of the solar flux retrievals
    flux_value : array
        the observed solar flux

    """
    try:
        with open(fname, 'r', newline='') as txtfile:
            # skip the first two lines
            for i in range(2):
                next(txtfile)

            # first count the lines
            reader = csv.DictReader(
                txtfile, delimiter=' ', skipinitialspace=True, fieldnames=[
                    'fluxdate', 'fluxtime', 'fluxjulian', 'fluxcarrington',
                    'fluxobsflux', 'fluxadjflux', 'fluxursi'])
            nrows = sum(1 for row in reader)

            flux_value = np.empty(nrows, dtype=float)

            # now read the data
            txtfile.seek(0)

            # skip the first two lines
            for i in range(2):
                next(txtfile)

            reader = csv.DictReader(
                txtfile, delimiter=' ', skipinitialspace=True, fieldnames=[
                    'fluxdate', 'fluxtime', 'fluxjulian', 'fluxcarrington',
                    'fluxobsflux', 'fluxadjflux', 'fluxursi'])

            flux_datetime = list()
            for i, row in enumerate(reader):
                flux_datetime.append(datetime.datetime.strptime(
                    row['fluxdate']+row['fluxtime'], '%Y%m%d%H%M%S'))
                flux_value[i] = float(row['fluxobsflux'])

            txtfile.close()

            return flux_datetime, flux_value

    except EnvironmentError as ee:
        warn(str(ee))
        warn('Unable to read file '+fname)
        return None, None



"""
pyrad.graph.plot_timeseries
===========================

Functions to plot Pyrad datasets

.. autosummary::
    :toctree: generated/

    plot_timeseries
    plot_timeseries_comp
    plot_monitoring_ts
    plot_intercomp_scores_ts
    plot_ml_ts
    plot_sun_retrieval_ts

"""

from warnings import warn

import numpy as np

import matplotlib as mpl
mpl.use('Agg')

# Increase a bit font size
mpl.rcParams.update({'font.size': 16})
mpl.rcParams.update({'font.family':  "sans-serif"})

import matplotlib.dates as mdates
import matplotlib.pyplot as plt

import pyart


def plot_timeseries(tvec, data_list, fname_list, labelx='Time [UTC]',
                    labely='Value', labels=['Sensor'], title='Time Series',
                    period=0, timeformat=None, colors=None, linestyles=None,
                    markers=None, ymin=None, ymax=None, dpi=72):
    """
    plots a time series

    Parameters
    ----------
    tvec : datetime object
        time of the time series
    data_list : list of float array
        values of the time series
    fname_list : list of str
        list of names of the files where to store the plot
    labelx : str
        The label of the X axis
    labely : str
        The label of the Y axis
    labels : array of str
        The label of the legend
    title : str
        The figure title
    period : float
        measurement period in seconds used to compute accumulation. If 0 no
        accumulation is computed
    timeformat : str
        Specifies the tvec and time format on the x axis
    colors : array of str
        Specifies the colors of each line
    linestyles : array of str
        Specifies the line style of each line
    markers: array of str
        Specify the markers to be used for each line
    ymin, ymax: float
        Lower/Upper limit of y axis
    dpi : int
        dots per inch

    Returns
    -------
    fname_list : list of str
        list of names of the created plots

    History
    --------
    201?.??.?? -fvj- creation
    2017.08.21 -jgr- modified margins and grid + minor graphical updates
    2018.03.05 -jgr- added x-limit of x axis to avoid unwanted error messages

    """
    if period > 0:
        for i, data in enumerate(data_list):
            data *= (period/3600.)
            data_list[i] = np.ma.cumsum(data)

    fig, ax = plt.subplots(figsize=[10, 6], dpi=dpi)

    lab = None
    col = None
    lstyle = '--'
    marker = 'o'

    for i, data in enumerate(data_list):
        if labels is not None:
            lab = labels[i]
        if colors is not None:
            col = colors[i]
        if linestyles is not None:
            lstyle = linestyles[i]
        if markers is not None:
            marker = markers[i]
        ax.plot(tvec, data, label=lab, color=col, linestyle=lstyle,
                marker=marker)

    ax.set_title(title)
    ax.set_xlabel(labelx)
    ax.set_ylabel(labely)
    ax.set_ylim(bottom=ymin, top=ymax)
    ax.set_xlim([tvec[0], tvec[-1]])

    # Turn on the grid
    ax.grid()

    if timeformat is not None:
        ax.xaxis.set_major_formatter(mdates.DateFormatter(timeformat))

    # rotates and right aligns the x labels, and moves the bottom of the
    # axes up to make room for them
    fig.autofmt_xdate()

    # Make a tight layout
    fig.tight_layout()

    for fname in fname_list:
        fig.savefig(fname, dpi=dpi)
    plt.close(fig)

    return fname_list


def plot_timeseries_comp(date1, value1, date2, value2, fname_list,
                         labelx='Time [UTC]', labely='Value',
                         label1='Sensor 1', label2='Sensor 2',
                         titl='Time Series Comparison', period1=0, period2=0,
                         ymin=None, ymax=None, dpi=72):
    """
    plots 2 time series in the same graph

    Parameters
    ----------
    date1 : datetime object
        time of the first time series
    value1 : float array
        values of the first time series
    date2 : datetime object
        time of the second time series
    value2 : float array
        values of the second time series
    fname_list : list of str
        list of names of the files where to store the plot
    labelx : str
        The label of the X axis
    labely : str
        The label of the Y axis
    label1, label2 : str
        legend label for each time series
    titl : str
        The figure title
     period1, period2 : float
        measurement period in seconds used to compute accumulation. If 0 no
        accumulation is computed
    dpi : int
        dots per inch
    ymin, ymax : float
        The limits of the Y-axis. None will keep the default limit.

    Returns
    -------
    fname_list : list of str
        list of names of the created plots

    History
    --------
    201?.??.?? -fvj- created
    2017.08.21 -jgr- changed some graphical aspects

    """
    if (period1 > 0) and (period2 > 0):
        # TODO: document this and check (sometimes artefacts)
        value1 *= (period1/3600.)
        value1 = np.ma.cumsum(value1)

        value2 *= (period2/3600.)
        value2 = np.ma.cumsum(value2)

    fig, ax = plt.subplots(figsize=[10, 6.5], dpi=dpi)
    ax.plot(date1, value1, 'b', label=label1, linestyle='--', marker='o')
    ax.plot(date2, value2, 'r', label=label2, linestyle='--', marker='s')
    ax.legend(loc='best')
    ax.set_xlabel(labelx)
    ax.set_ylabel(labely)
    ax.set_title(titl)

    ax.grid()

    ax.set_ylim(bottom=ymin, top=ymax)
    ax.set_xlim([date2[0], date2[-1]])

    # rotates and right aligns the x labels, and moves the bottom of the
    # axes up to make room for them
    fig.autofmt_xdate()

    # Make a tight layout
    fig.tight_layout()

    for fname in fname_list:
        fig.savefig(fname, dpi=dpi)
    plt.close(fig)

    return fname_list


def plot_monitoring_ts(date, np_t, cquant, lquant, hquant, field_name,
                       fname_list, ref_value=None, vmin=None, vmax=None,
                       np_min=0, labelx='Time [UTC]', labely='Value',
                       titl='Time Series', dpi=72):
    """
    plots a time series of monitoring data

    Parameters
    ----------
    date : datetime object
        time of the time series
    np_t : int array
        number of points
    cquant, lquant, hquant : float array
        values of the central, low and high quantiles
    field_name : str
        name of the field
    fname_list : list of str
        list of names of the files where to store the plot
    ref_value : float
        the reference value
    vmin, vmax : float
        The limits of the y axis
    np_min : int
        minimum number of points to consider the sample plotable
    labelx : str
        The label of the X axis
    labely : str
        The label of the Y axis
    titl : str
        The figure title
    dpi : int
        dots per inch

    Returns
    -------
    fname_list : list of str
        list of names of the created plots

    """
    vmin_pyart, vmax_pyart = pyart.config.get_field_limits(field_name)
    if vmin is None:
        vmin = vmin_pyart
    if vmax is None:
        vmax = vmax_pyart

    # plot only valid data (but keep first and last date)
    date2 = np.array(date)
    isvalid = np.logical_not(np.ma.getmaskarray(cquant))
    if np_min > 0:
        has_np = np_t > np_min
        isvalid = np.logical_and(isvalid, has_np)

    cquant_plt = cquant[isvalid]
    lquant_plt = lquant[isvalid]
    hquant_plt = hquant[isvalid]
    date_plt = date2[isvalid]
    if not isvalid[0]:
        cquant_plt = np.ma.append(np.ma.masked, cquant_plt)
        lquant_plt = np.ma.append(np.ma.masked, lquant_plt)
        hquant_plt = np.ma.append(np.ma.masked, hquant_plt)
        date_plt = np.ma.append(date2[0], date_plt)
    if not isvalid[-1]:
        cquant_plt = np.ma.append(cquant_plt, np.ma.masked)
        lquant_plt = np.ma.append(lquant_plt, np.ma.masked)
        hquant_plt = np.ma.append(hquant_plt, np.ma.masked)
        date_plt = np.ma.append(date_plt, date2[-1])

    fig = plt.figure(figsize=[15, 13], dpi=dpi)

    ax = fig.add_subplot(2, 1, 1)
    ax.plot(date_plt, cquant_plt, 'x-')
    ax.plot(date_plt, lquant_plt, 'rx-')
    ax.plot(date_plt, hquant_plt, 'rx-')
    if ref_value is not None:
        ax.plot(date_plt, np.zeros(len(date_plt))+ref_value, 'k--')
    ax.set_ylabel(labely)
    ax.set_title(titl)
    ax.set_ylim([vmin, vmax])

    # tight x axis
    ax.autoscale(enable=True, axis='x', tight=True)
    ax.grid(True)

    ax = fig.add_subplot(2, 1, 2)
    ax.plot(date, np_t, 'x-')

    if np_min is not None:
        ax.plot(date, np.zeros(len(date))+np_min, 'k--')

    ax.set_ylabel('Number of Samples')
    ax.set_xlabel(labelx)

    # rotates and right aligns the x labels, and moves the bottom of the
    # axes up to make room for them
    fig.autofmt_xdate()

    # tight x axis
    ax.autoscale(enable=True, axis='x', tight=True)

    for fname in fname_list:
        fig.savefig(fname, dpi=dpi)
    plt.close(fig)

    return fname_list


def plot_intercomp_scores_ts(date_vec, np_vec, meanbias_vec, medianbias_vec,
                             quant25bias_vec, quant75bias_vec, modebias_vec,
                             corr_vec, slope_vec, intercep_vec,
                             intercep_slope1_vec, fname_list, ref_value=0.,
                             np_min=0, corr_min=0.,
                             labelx='Time UTC',
                             titl='RADAR001-RADAR002 intercomparison',
                             dpi=72):
    """
    plots a time series of radar intercomparison scores

    Parameters
    ----------
    date_vec : datetime object
        time of the time series
    np_vec : int array
        number of points
    meanbias_vec, medianbias_vec, modebias_vec : float array
        mean, median and mode bias
    quant25bias_vec, quant75bias_vec: 25th and 75th percentile of the bias
    corr_vec : float array
        correlation
    slope_vec, intercep_vec : float array
        slope and intercep of a linear regression
    intercep_slope1_vec : float
        the intercep point of a inear regression of slope 1
    ref_value : float
        the reference value
    np_min : int
        The minimum number of points to consider the result valid
    corr_min : float
        The minimum correlation to consider the results valid
    labelx : str
        The label of the X axis
    titl : str
        The figure title

    Returns
    -------
    fname_list : list of str
        list of names of the created plots

    """
    # plot only valid data (but keep first and last date)
    date2 = np.array(date_vec)
    isvalid = np.logical_not(np.ma.getmaskarray(meanbias_vec))
    isvalid_corr = np.logical_not(np.ma.getmaskarray(corr_vec))
    if np_min > 0:
        has_np = np_vec > np_min
        isvalid = np.logical_and(isvalid, has_np)
    if corr_min > 0:
        has_corr_min = corr_vec > corr_min
        isvalid = np.logical_and(isvalid, has_corr_min)

    meanbias_plt = meanbias_vec[isvalid]
    medianbias_plt = medianbias_vec[isvalid]
    quant25bias_plt = quant25bias_vec[isvalid]
    quant75bias_plt = quant75bias_vec[isvalid]
    modebias_plt = modebias_vec[isvalid]
    intercep_plt = intercep_slope1_vec[isvalid]
    corr_plt = corr_vec[isvalid_corr]
    date_corr = date2[isvalid_corr]
    date_plt = date2[isvalid]
    if not isvalid[0]:
        meanbias_plt = np.ma.append(np.ma.masked, meanbias_plt)
        medianbias_plt = np.ma.append(np.ma.masked, medianbias_plt)
        quant25bias_plt = np.ma.append(np.ma.masked, quant25bias_plt)
        quant75bias_plt = np.ma.append(np.ma.masked, quant75bias_plt)
        modebias_plt = np.ma.append(np.ma.masked, modebias_plt)
        intercep_plt = np.ma.append(np.ma.masked, intercep_plt)
        date_plt = np.ma.append(date2[0], date_plt)
    if not isvalid[-1]:
        meanbias_plt = np.ma.append(meanbias_plt, np.ma.masked)
        medianbias_plt = np.ma.append(medianbias_plt, np.ma.masked)
        quant25bias_plt = np.ma.append(quant25bias_plt, np.ma.masked)
        quant75bias_plt = np.ma.append(quant75bias_plt, np.ma.masked)
        modebias_plt = np.ma.append(modebias_plt, np.ma.masked)
        intercep_plt = np.ma.append(intercep_plt, np.ma.masked)
        date_plt = np.ma.append(date_plt, date2[-1])

    if not isvalid_corr[0]:
        corr_plt = np.ma.append(np.ma.masked, corr_plt)
        date_corr = np.ma.append(date2[0], date_corr)
    if not isvalid_corr[-1]:
        corr_plt = np.ma.append(corr_plt, np.ma.masked)
        date_corr = np.ma.append(date_corr, date2[-1])

    fig = plt.figure(figsize=[10, 20], dpi=dpi)

    ax = fig.add_subplot(4, 1, 1)
    ax.plot(date_plt, medianbias_plt, 'bx-', label='median')
    ax.plot(date_plt, meanbias_plt, 'rx-', label='mean')
    ax.plot(date_plt, modebias_plt, 'gx-', label='mode')
    ax.plot(date_plt, intercep_plt, 'yx-', label='intercep of slope 1 LR')
    if ref_value is not None:
        ax.plot(date_plt, np.zeros(len(date_plt))+ref_value, 'k--')
    # plt.legend(loc='best')
    ax.set_ylabel('bias [dB]')
    ax.set_title(titl)
    ax.set_ylim([-5., 5.])

    # tight x axis
    ax.autoscale(enable=True, axis='x', tight=True)
    ax.grid(True)

    ax = fig.add_subplot(4, 1, 2)
    ax.plot(date_plt, medianbias_plt, 'bx-', label='median')
    ax.plot(date_plt, quant25bias_plt, 'rx-', label='25-percentile')
    ax.plot(date_plt, quant75bias_plt, 'rx-', label='75-percentile')
    if ref_value is not None:
        ax.plot(date_plt, np.zeros(len(date_plt))+ref_value, 'k--')
    # plt.legend(loc='best')
    ax.set_ylabel('bias [dB]')
    ax.set_ylim([-5., 5.])

    # tight x axis
    ax.autoscale(enable=True, axis='x', tight=True)
    ax.grid(True)

    ax = fig.add_subplot(4, 1, 3)
    ax.plot(date_corr, corr_plt, 'bx-')

    if corr_min > 0:
        ax.plot(date_corr, np.zeros(len(date_corr))+corr_min, 'k--')

    ax.set_ylabel('correlation')
    ax.set_ylim([0., 1.])

    # tight x axis
    ax.autoscale(enable=True, axis='x', tight=True)
    ax.grid(True)

    ax = fig.add_subplot(4, 1, 4)
    ax.plot(date2, np_vec, 'bx-')

    if np_min > 0:
        ax.plot(date2, np.zeros(len(date2))+np_min, 'k--')

    ax.set_ylabel('Number of Samples')
    ax.set_xlabel(labelx)

    # tight x axis
    ax.autoscale(enable=True, axis='x', tight=True)

    # rotates and right aligns the x labels, and moves the bottom of the
    # axes up to make room for them
    fig.autofmt_xdate()

    for fname in fname_list:
        fig.savefig(fname, dpi=dpi)
    plt.close(fig)

    return fname_list


def plot_ml_ts(dt_ml_arr, ml_top_avg_arr, ml_top_std_arr, thick_avg_arr,
               thick_std_arr, nrays_valid_arr, nrays_total_arr, fname_list,
               labelx='Time UTC', titl='Melting layer time series', dpi=72):
    """
    plots a time series of melting layer data

    Parameters
    ----------
    dt_ml_arr : datetime object
        time of the time series
    np_vec : int array
        number of points
    meanbias_vec, medianbias_vec, modebias_vec : float array
        mean, median and mode bias
    quant25bias_vec, quant75bias_vec: 25th and 75th percentile of the bias
    corr_vec : float array
        correlation
    slope_vec, intercep_vec : float array
        slope and intercep of a linear regression
    intercep_slope1_vec : float
        the intercep point of a inear regression of slope 1
    ref_value : float
        the reference value
    np_min : int
        The minimum number of points to consider the result valid
    corr_min : float
        The minimum correlation to consider the results valid
    labelx : str
        The label of the X axis
    titl : str
        The figure title

    Returns
    -------
    fname_list : list of str
        list of names of the created plots

    """
    fig = plt.figure(figsize=[10, 15], dpi=dpi)

    ax = fig.add_subplot(3, 1, 1)
    ax.plot(dt_ml_arr, ml_top_avg_arr, 'bx-', label='avg')
    ax.plot(dt_ml_arr, ml_top_avg_arr+ml_top_std_arr, 'rx-', label='avg+std')
    ax.plot(dt_ml_arr, ml_top_avg_arr-ml_top_std_arr, 'rx-', label='avg-std')
    # plt.legend(loc='best')
    ax.set_ylabel('Top height [m MSL]')
    ax.set_title(titl)
    ax.set_ylim([0., 6000.])
    ax.set_xlim([dt_ml_arr[0], dt_ml_arr[-1]])

    # tight x axis
    ax.autoscale(enable=True, axis='x', tight=True)
    ax.grid(True)

    ax = fig.add_subplot(3, 1, 2)
    ax.plot(dt_ml_arr, thick_avg_arr, 'bx-', label='avg')
    ax.plot(dt_ml_arr, thick_avg_arr+thick_std_arr, 'rx-', label='avg+std')
    ax.plot(dt_ml_arr, thick_avg_arr-thick_std_arr, 'rx-', label='avg-std')
    # plt.legend(loc='best')
    ax.set_ylabel('Thickness [m]')
    ax.set_ylim([0., 3000.])
    ax.set_xlim([dt_ml_arr[0], dt_ml_arr[-1]])

    # tight x axis
    ax.autoscale(enable=True, axis='x', tight=True)
    ax.grid(True)

    ax = fig.add_subplot(3, 1, 3)
    ax.plot(dt_ml_arr, nrays_valid_arr, 'bx-', label='N valid rays')
    ax.plot(dt_ml_arr, nrays_total_arr, 'rx-', label='rays total')
    # plt.legend(loc='best')
    ax.set_ylabel('Rays')
    ax.set_xlabel(labelx)
    ax.set_ylim([0, np.max(nrays_total_arr)+5])
    ax.set_xlim([dt_ml_arr[0], dt_ml_arr[-1]])

    # tight x axis
    ax.autoscale(enable=True, axis='x', tight=True)
    ax.grid(True)

    # rotates and right aligns the x labels, and moves the bottom of the
    # axes up to make room for them
    fig.autofmt_xdate()

    for fname in fname_list:
        fig.savefig(fname, dpi=dpi)
    plt.close(fig)

    return fname_list


def plot_sun_retrieval_ts(sun_retrieval, data_type, fname_list, labelx='Date',
                          titl='Sun retrieval Time Series', dpi=72):
    """
    plots sun retrieval time series series

    Parameters
    ----------
    sun_retrieval : tuple
        tuple containing the retrieved parameters
    data_type : str
        parameter to be plotted
    fname_list : list of str
        list of names of the files where to store the plot
    labelx : str
        the x label
    titl : str
        the title of the plot
    dpi : int
        dots per inch

    Returns
    -------
    fname_list : list of str
        list of names of the created plots

    """
    value_std = None
    ref = None
    date = sun_retrieval[1]
    if data_type == 'nhits_h':
        value = sun_retrieval[2]
        labely = 'Number of sun hits H channel'
        vmin = 0
        vmax = np.max(sun_retrieval[2])+1
    elif data_type == 'el_width_h':
        value = sun_retrieval[3]
        labely = 'Elevation beamwidth H channel (Deg)'
        vmin = 0.
        vmax = 4.
    elif data_type == 'az_width_h':
        value = sun_retrieval[4]
        labely = 'Azimuth beamwidth H channel (Deg)'
        vmin = 0.
        vmax = 4.
    elif data_type == 'el_bias_h':
        value = sun_retrieval[5]
        ref = np.zeros(len(value))
        labely = 'Elevation pointing bias H channel (Deg)'
        vmin = -2.
        vmax = 2.
    elif data_type == 'az_bias_h':
        value = sun_retrieval[6]
        ref = np.zeros(len(value))
        labely = 'Azimuth pointing bias H channel (Deg)'
        vmin = -2.
        vmax = 2.
    elif data_type == 'dBm_sun_est':
        value = sun_retrieval[7]
        value_std = sun_retrieval[8]
        labely = 'Sun Power H channel (dBm)'
        vmin = -110.
        vmax = -90.
    elif data_type == 'rx_bias_h':
        value = (10.*np.ma.log10(sun_retrieval[9]) -
                 10.*np.ma.log10(sun_retrieval[21]))
        value_std = sun_retrieval[8]
        ref = np.zeros(len(value))
        labely = 'Receiver bias H channel (dB)'
        vmin = -5.
        vmax = 5.
    elif data_type == 'sf_h':
        value = 10.*np.ma.log10(sun_retrieval[9])
        # value_std = sun_retrieval[8]
        ref = 10.*np.ma.log10(sun_retrieval[21])
        labely = 'Observed solar flux H channel (dB(sfu))'
        vmin = 15.
        vmax = 30.
    elif data_type == 'nhits_v':
        value = sun_retrieval[10]
        labely = 'Number of sun hits V channel'
        vmin = 0
        vmax = np.max(sun_retrieval[10])+1
    elif data_type == 'el_width_v':
        value = sun_retrieval[11]
        labely = 'Elevation beamwidth V channel (Deg)'
        vmin = 0.
        vmax = 4.
    elif data_type == 'az_width_v':
        value = sun_retrieval[12]
        labely = 'Azimuth beamwidth V channel (Deg)'
        vmin = 0.
        vmax = 4.
    elif data_type == 'el_bias_v':
        value = sun_retrieval[13]
        ref = np.zeros(len(value))
        labely = 'Elevation pointing bias V channel (Deg)'
        vmin = -2.
        vmax = 2.
    elif data_type == 'az_bias_v':
        value = sun_retrieval[14]
        ref = np.zeros(len(value))
        labely = 'Azimuth pointing bias V channel (Deg)'
        vmin = -2.
        vmax = 2.
    elif data_type == 'dBmv_sun_est':
        value = sun_retrieval[15]
        value_std = sun_retrieval[16]
        labely = 'Sun Power V channel (dBm)'
        vmin = -110.
        vmax = -90.
    elif data_type == 'rx_bias_v':
        value = (10.*np.ma.log10(sun_retrieval[17]) -
                 10.*np.ma.log10(sun_retrieval[21]))
        value_std = sun_retrieval[16]
        ref = np.zeros(len(value))
        labely = 'Receiver bias V channel (dB)'
        vmin = -5.
        vmax = 5.
    elif data_type == 'sf_v':
        value = 10.*np.ma.log10(sun_retrieval[17])
        # value_std = sun_retrieval[16]
        ref = 10.*np.ma.log10(sun_retrieval[21])
        labely = 'Observed solar flux V channel (dB(sfu))'
        vmin = 15.
        vmax = 30.
    elif data_type == 'nhits_zdr':
        value = sun_retrieval[18]
        labely = 'Number of sun hits ZDR'
        vmin = 0
        vmax = np.max(sun_retrieval[18])+1
    elif data_type == 'ZDR_sun_est':
        value = sun_retrieval[19]
        value_std = sun_retrieval[20]
        ref = np.zeros(len(value))
        labely = 'Sun ZDR (dB)'
        vmin = -2.
        vmax = 2.

    mask = np.ma.getmaskarray(value)
    if mask.all():
        warn('Unable to create figure '+' '.join(fname_list) +
             '. No valid data')
        return None

    # plot only valid data (but keep first and last date)
    isvalid = np.logical_not(mask)
    date2 = np.array(date)

    value_plt = value[isvalid]
    date_plt = date2[isvalid]
    if not isvalid[0]:
        value_plt = np.ma.append(np.ma.masked, value_plt)
        date_plt = np.ma.append(date2[0], date_plt)
    if not isvalid[-1]:
        value_plt = np.ma.append(value_plt, np.ma.masked)
        date_plt = np.ma.append(date_plt, date2[-1])

    fig, ax = plt.subplots(figsize=[10, 6], dpi=dpi)
    ax.plot(date_plt, value_plt, 'x-')
    if value_std is not None:
        value_std_plt = value_std[isvalid]
        if not isvalid[0]:
            value_std_plt = np.ma.append(np.ma.masked, value_std_plt)
        if not isvalid[-1]:
            value_std_plt = np.ma.append(value_std_plt, np.ma.masked)

        ax.plot(date_plt, value_plt+value_std_plt, 'rx-')
        ax.plot(date_plt, value_plt-value_std_plt, 'rx-')
    if ref is not None:
        ref_plt = ref[isvalid]
        if not isvalid[0]:
            ref_plt = np.ma.append(ref[0], ref_plt)
        if not isvalid[-1]:
            ref_plt = np.ma.append(ref_plt, ref[-1])
        ax.plot(date_plt, ref_plt, 'k--')
    ax.set_xlabel(labelx)
    ax.set_ylabel(labely)
    ax.set_title(titl)
    ax.set_ylim([vmin, vmax])
    ax.set_xlim([date_plt[0], date_plt[-1]])
    # tight x axis
    ax.autoscale(enable=True, axis='x', tight=True)
    ax.grid(True)

    # rotates and right aligns the x labels, and moves the bottom of the
    # axes up to make room for them
    fig.autofmt_xdate()

    for fname in fname_list:
        fig.savefig(fname, dpi=dpi)
    plt.close(fig)

    return fname_list



"""
pyrad.proc.process_Doppler
===========================

Functions for processing Doppler related parameters

.. autosummary::
    :toctree: generated/

    process_turbulence
    process_dealias_fourdd
    process_dealias_region_based
    process_dealias_unwrap_phase
    process_radial_velocity
    process_wind_vel
    process_windshear
    process_vad

"""

from copy import deepcopy
from warnings import warn

import numpy as np

import pyart

try:
    import pytda
    _PYTDA_AVAILABLE = True
except ImportError:
    _PYTDA_AVAILABLE = False

from ..io.io_aux import get_datatype_fields, get_fieldname_pyart


def process_turbulence(procstatus, dscfg, radar_list=None):
    """
    Computes turbulence from the Doppler spectrum width and reflectivity using
    the PyTDA package

    Parameters
    ----------
    procstatus : int
        Processing status: 0 initializing, 1 processing volume,
        2 post-processing
    dscfg : dictionary of dictionaries
        data set configuration. Accepted Configuration Keywords::

        datatype : string. Dataset keyword
            The input data type
        radius : float. Dataset keyword
            Search radius for calculating Eddy Dissipation Rate (EDR).
            Default 2
        split_cut : Bool. Dataset keyword
            Set to True for split-cut volumes. Default False
        max_split_cut : Int. Dataset keyword
            Total number of tilts that are affected by split cuts. Only
            relevant if split_cut=True. Default 2
        xran, yran : float array. Dataset keyword
            Spatial range in X,Y to consider. Default [-100, 100] for both
            X and Y
        beamwidth : Float. Dataset keyword
            Radar beamwidth. Default None. If None it will be obtained from
            the radar object metadata. If cannot be obtained defaults to 1
            deg.
        compute_gate_pos : Bool. Dataset keyword
            If True the gate position is going to be computed in PyTDA.
            Otherwise the position from the radar object is used. Default
            False
        verbose : Bool. Dataset keyword
            True for verbose output. Default False

    radar_list : list of Radar objects
        Optional. list of radar objects

    Returns
    -------
    new_dataset : dict
        dictionary containing the output
    ind_rad : int
        radar index

    """
    if not _PYTDA_AVAILABLE:
        warn('PyTDA package not available. Unable to compute turbulence')
        return None, None

    if procstatus != 1:
        return None, None

    width_field = None
    refl_field = None
    for datatypedescr in dscfg['datatype']:
        radarnr, _, datatype, _, _ = get_datatype_fields(datatypedescr)
        if datatype in ('dBuZ', 'dBZ', 'dBZc', 'dBuZv', 'dBZv', 'dBZvc'):
            refl_field = get_fieldname_pyart(datatype)
        if datatype in ('W', 'Wv', 'Wu', 'Wvu'):
            width_field = get_fieldname_pyart(datatype)

    if width_field is None or refl_field is None:
        warn('Reflectivity and spectrum width fields required'
             ' to estimate turbulence')
        return None, None

    ind_rad = int(radarnr[5:8])-1
    if radar_list[ind_rad] is None:
        warn('No valid radar')
        return None, None
    radar = radar_list[ind_rad]

    if width_field not in radar.fields or refl_field not in radar.fields:
        warn('Unable to compute turbulence. Missing data')
        return None, None

    # user defined parameters
    radius = dscfg.get('radius', 2.)
    split_cut = dscfg.get('split_cut', False)
    xran = dscfg.get('xran', [-100., 100.])
    yran = dscfg.get('yran', [-100., 100.])
    max_split_cut = dscfg.get('max_split_cut', 2)
    beamwidth = dscfg.get('beamwidth', None)
    verbose = dscfg.get('verbose', False)
    compute_gate_pos = dscfg.get('compute_gate_pos', False)

    if beamwidth is None:
        if (radar.instrument_parameters is not None and
                'radar_beam_width_h' in radar.instrument_parameters):
            beamwidth = radar.instrument_parameters[
                'radar_beam_width_h']['data'][0]
        else:
            warn('Unknown radar beamwidth. Default 1 deg will be used')
            beamwidth = 1

    rng_res = radar.range['data'][1]-radar.range['data'][0]/1000.

    radar_out = deepcopy(radar)
    radar_out.fields = dict()
    radar_out.add_field(refl_field, deepcopy(radar.fields[refl_field]))
    radar_out.add_field(width_field, deepcopy(radar.fields[width_field]))
    radar_out.fields[refl_field]['data'][
        np.ma.getmaskarray(radar_out.fields[refl_field]['data'])] = -32768
    radar_out.fields[width_field]['data'][
        np.ma.getmaskarray(radar_out.fields[width_field]['data'])] = -32768

    radar_out.fields[refl_field]['_FillValue'] = -32768
    radar_out.fields[width_field]['_FillValue'] = -32768

    if radar_out.scan_type == 'ppi':
        pytda.calc_turb_vol(
            radar_out, radius=radius, split_cut=split_cut, xran=xran,
            yran=yran, verbose=verbose, name_dz=refl_field,
            name_sw=width_field, turb_name='turbulence',
            max_split_cut=max_split_cut, use_ntda=True, beamwidth=beamwidth,
            gate_spacing=rng_res, compute_gate_pos=compute_gate_pos)
    elif radar_out.scan_type == 'rhi':
        pytda.calc_turb_rhi(
            radar_out, radius=radius, verbose=verbose, name_dz=refl_field,
            name_sw=width_field, turb_name='turbulence',
            use_ntda=True, beamwidth=beamwidth, gate_spacing=rng_res,
            compute_gate_pos=compute_gate_pos)
    else:
        warn('Radar volume of type '+radar_out.scan_type +
             '. Only volumes of type PPI or RHI are allowed')
        return None, None

    del radar_out.fields[refl_field]
    del radar_out.fields[width_field]

    radar_out.fields['turbulence']['data'] = np.ma.masked_values(
        radar_out.fields['turbulence']['data'], -32768.)

    # prepare for exit
    new_dataset = {'radar_out': radar_out}

    return new_dataset, ind_rad


def process_dealias_fourdd(procstatus, dscfg, radar_list=None):
    """
    Dealiases the Doppler velocity field using the 4DD technique
    from Curtis and Houze, 2001

    Parameters
    ----------
    procstatus : int
        Processing status: 0 initializing, 1 processing volume,
        2 post-processing
    dscfg : dictionary of dictionaries
        data set configuration. Accepted Configuration Keywords::

        datatype : string. Dataset keyword
            The input data type
        filt : int. Dataset keyword
            Flag controlling Bergen and Albers filter, 1 = yes, 0 = no.
        sign : int. Dataset keyword
            Sign convention which the radial velocities in the volume created
            from the sounding data will will. This should match the
            convention used in the radar data. A value of 1 represents when
            positive values velocities are towards the radar, -1 represents
            when negative velocities are towards the radar.


    radar_list : list of Radar objects
        Optional. list of radar objects

    Returns
    -------
    new_dataset : dict
        dictionary containing the output
    ind_rad : int
        radar index

    """
    if procstatus != 1:
        return None, None

    radarnr, _, datatype, _, _ = get_datatype_fields(dscfg['datatype'][0])
    vel_field = get_fieldname_pyart(datatype)

    ind_rad = int(radarnr[5:8])-1
    if radar_list[ind_rad] is None:
        warn('No valid radar')
        return None, None
    radar = radar_list[ind_rad]

    if vel_field not in radar.fields:
        warn('Unable to correct Doppler aliasing. Missing data')
        return None, None

    corr_vel_field = 'dealiased_'+vel_field

    if not dscfg['initialized']:
        # Use phase unwraping method to obtain first guess

        # get user parameters
        interval_splits = dscfg.get('interval_splits', 3)
        skip_between_rays = dscfg.get('skip_between_rays', 100)
        skip_along_ray = dscfg.get('skip_along_ray', 100)
        centered = dscfg.get('centered', True)

        corr_vel_dict = pyart.correct.dealias_region_based(
            radar, ref_vel_field=None, interval_splits=interval_splits,
            interval_limits=None, skip_between_rays=skip_between_rays,
            skip_along_ray=skip_along_ray, centered=centered,
            nyquist_vel=None, check_nyquist_uniform=True, gatefilter=False,
            rays_wrap_around=None, keep_original=False, set_limits=False,
            vel_field=vel_field, corr_vel_field=corr_vel_field)

        dscfg['initialized'] = 1
    else:
        # get user parameters
        filt = dscfg.get('filt', 1)
        sign = dscfg.get('sign', 1)
        keep_mask = dscfg.get('keep_mask', True)

        last_radar = dscfg['global_data']

        corr_vel_dict = pyart.correct.dealias_fourdd(
            radar, last_radar=last_radar, sonde_profile=None,
            gatefilter=False, filt=filt, rsl_badval=131072.0,
            keep_original=False, set_limits=False, vel_field=vel_field,
            corr_vel_field=corr_vel_field, last_vel_field=corr_vel_field,
            debug=False, sign=sign)

        if keep_mask:
            mask = np.ma.getmaskarray(radar.fields[vel_field]['data'])
            corr_vel_dict['data'] = np.ma.masked_where(
                mask, corr_vel_dict['data'])

    # prepare for exit
    radar_out = deepcopy(radar)
    radar_out.fields = dict()
    radar_out.add_field(corr_vel_field, corr_vel_dict)
    new_dataset = {'radar_out': radar_out}

    # keep current corrected Doppler velocity field in memory
    dscfg['global_data'] = radar_out

    return new_dataset, ind_rad


def process_dealias_region_based(procstatus, dscfg, radar_list=None):
    """
    Dealiases the Doppler velocity field using a region based algorithm

    Parameters
    ----------
    procstatus : int
        Processing status: 0 initializing, 1 processing volume,
        2 post-processing
    dscfg : dictionary of dictionaries
        data set configuration. Accepted Configuration Keywords::

        datatype : string. Dataset keyword
            The input data type
        interval_splits : int, optional
            Number of segments to split the nyquist interval into when finding
            regions of similar velocity.  More splits creates a larger number
            of initial regions which takes longer to process but may result in
            better dealiasing.  The default value of 3 seems to be a good
            compromise between performance and artifact free dealiasing. This
            value is not used if the interval_limits parameter is not None.
        skip_between_rays, skip_along_ray : int, optional
            Maximum number of filtered gates to skip over when joining
            regions, gaps between region larger than this will not be
            connected. Parameters specify the maximum number of filtered gates
            between and along a ray. Set these parameters to 0 to disable
            unfolding across filtered gates.
        centered : bool, optional
            True to apply centering to each sweep after the dealiasing
            algorithm so that the average number of unfolding is near 0. False
            does not apply centering which may results in individual sweeps
            under or over folded by the nyquist interval.
    radar_list : list of Radar objects
        Optional. list of radar objects

    Returns
    -------
    new_dataset : dict
        dictionary containing the output
    ind_rad : int
        radar index

    """
    if procstatus != 1:
        return None, None

    radarnr, _, datatype, _, _ = get_datatype_fields(dscfg['datatype'][0])
    vel_field = get_fieldname_pyart(datatype)

    ind_rad = int(radarnr[5:8])-1
    if radar_list[ind_rad] is None:
        warn('No valid radar')
        return None, None
    radar = radar_list[ind_rad]

    if vel_field not in radar.fields:
        warn('Unable to correct Doppler aliasing. Missing data')
        return None, None

    corr_vel_field = 'dealiased_'+vel_field

    # get user parameters
    interval_splits = dscfg.get('interval_splits', 3)
    skip_between_rays = dscfg.get('skip_between_rays', 100)
    skip_along_ray = dscfg.get('skip_along_ray', 100)
    centered = dscfg.get('centered', True)

    corr_vel_dict = pyart.correct.dealias_region_based(
        radar, ref_vel_field=None, interval_splits=interval_splits,
        interval_limits=None, skip_between_rays=skip_between_rays,
        skip_along_ray=skip_along_ray, centered=centered,
        nyquist_vel=None, check_nyquist_uniform=True, gatefilter=False,
        rays_wrap_around=None, keep_original=False, set_limits=False,
        vel_field=vel_field, corr_vel_field=corr_vel_field)

    # prepare for exit
    new_dataset = {'radar_out': deepcopy(radar)}
    new_dataset['radar_out'].fields = dict()
    new_dataset['radar_out'].add_field(corr_vel_field, corr_vel_dict)

    return new_dataset, ind_rad


def process_dealias_unwrap_phase(procstatus, dscfg, radar_list=None):
    """
    Dealiases the Doppler velocity field using multi-dimensional phase
    unwrapping

    Parameters
    ----------
    procstatus : int
        Processing status: 0 initializing, 1 processing volume,
        2 post-processing
    dscfg : dictionary of dictionaries
        data set configuration. Accepted Configuration Keywords::

        datatype : string. Dataset keyword
            The input data type
        unwrap_unit : {'ray', 'sweep', 'volume'}, optional
            Unit to unwrap independently.  'ray' will unwrap each ray
            individually, 'sweep' each sweep, and 'volume' will unwrap the
            entire volume in a single pass.  'sweep', the default, often gives
            superior results when the lower sweeps of the radar volume are
            contaminated by clutter. 'ray' does not use the gatefilter
            parameter and rays where gates ared masked will result in poor
            dealiasing for that ray.
    radar_list : list of Radar objects
        Optional. list of radar objects

    Returns
    -------
    new_dataset : dict
        dictionary containing the output
    ind_rad : int
        radar index

    """
    if procstatus != 1:
        return None, None

    radarnr, _, datatype, _, _ = get_datatype_fields(dscfg['datatype'][0])
    vel_field = get_fieldname_pyart(datatype)

    ind_rad = int(radarnr[5:8])-1
    if radar_list[ind_rad] is None:
        warn('No valid radar')
        return None, None
    radar = radar_list[ind_rad]

    if vel_field not in radar.fields:
        warn('Unable to correct Doppler aliasing. Missing data')
        return None, None

    corr_vel_field = 'dealiased_'+vel_field

    # get user parameters
    unwrap_unit = dscfg.get('unwrap_unit', 'sweep')

    corr_vel_dict = pyart.correct.dealias_unwrap_phase(
        radar, unwrap_unit=unwrap_unit, nyquist_vel=None,
        check_nyquist_uniform=True, gatefilter=False,
        rays_wrap_around=None, keep_original=False, set_limits=False,
        vel_field=vel_field, corr_vel_field=corr_vel_field, skip_checks=False)

    # prepare for exit
    new_dataset = {'radar_out': deepcopy(radar)}
    new_dataset['radar_out'].fields = dict()
    new_dataset['radar_out'].add_field(corr_vel_field, corr_vel_dict)

    return new_dataset, ind_rad


def process_radial_velocity(procstatus, dscfg, radar_list=None):
    """
    Estimates the radial velocity respect to the radar from the wind velocity

    Parameters
    ----------
    procstatus : int
        Processing status: 0 initializing, 1 processing volume,
        2 post-processing
    dscfg : dictionary of dictionaries
        data set configuration. Accepted Configuration Keywords::

        datatype : string. Dataset keyword
            The input data type
        latitude, longitude : float
            arbitrary coordinates [deg] from where to compute the radial
            velocity. If any of them is None it will be the radar position
        altitude : float
            arbitrary altitude [m MSL] from where to compute the radial
            velocity. If None it will be the radar altitude
    radar_list : list of Radar objects
        Optional. list of radar objects

    Returns
    -------
    new_dataset : dict
        dictionary containing the output
    ind_rad : int
        radar index

    """
    if procstatus != 1:
        return None, None

    v_speed_field = None
    h_speed_field = None
    h_dir_field = None
    for datatypedescr in dscfg['datatype']:
        radarnr, _, datatype, _, _ = get_datatype_fields(datatypedescr)
        if datatype == 'wind_vel_v':
            v_speed_field = get_fieldname_pyart(datatype)
        if datatype == 'WIND_SPEED':
            h_speed_field = get_fieldname_pyart(datatype)
        if datatype == 'WIND_DIRECTION':
            h_dir_field = get_fieldname_pyart(datatype)

    if h_speed_field is None or h_dir_field is None:
        warn('Horizontal wind speed and direction fields required'
             ' to estimate radial velocity')
        return None, None

    ind_rad = int(radarnr[5:8])-1
    if radar_list[ind_rad] is None:
        warn('No valid radar')
        return None, None
    radar = radar_list[ind_rad]

    if h_speed_field not in radar.fields or h_dir_field not in radar.fields:
        warn('Unable to estimate radial velocity. '
             'Missing horizontal wind')
        return None, None

    h_speed = radar.fields[h_speed_field]['data']
    h_dir = radar.fields[h_dir_field]['data']

    if v_speed_field is None or v_speed_field not in radar.fields:
        warn('Unknown vertical wind speed. Assumed 0')

        if v_speed_field is None:
            v_speed_field == 'vertical_wind_component'

        v_speed = np.ma.zeros((radar.nrays, radar.ngates))
    else:
        v_speed = radar.fields[v_speed_field]['data']

    # user defined parameters
    lat = dscfg.get('latitude', None)
    lon = dscfg.get('longitude', None)
    alt = dscfg.get('altitude', None)

    # get u and v wind components
    h_dir_rad = np.deg2rad(h_dir)
    speed_h_u = h_speed*np.sin(h_dir_rad)  # eastward component
    speed_h_v = h_speed*np.cos(h_dir_rad)  # northward component

    if lat is not None or lon is not None or alt is not None:
        # get antenna coordinates respect to new radar location
        if lat is None:
            lat = radar.latitude['data'][0]
        if lon is None:
            lon = radar.longitude['data'][0]
        if alt is None:
            alt = radar.altitude['data'][0]

        x, y = pyart.core.geographic_to_cartesian_aeqd(
            radar.gate_longitude['data'], radar.gate_latitude['data'], lon,
            lat)
        z = radar.gate_altitude['data'] - alt
        _, azimuths, elevations = pyart.core.cartesian_to_antenna(
            x, y, z)
        azi_2D_rad = np.deg2rad(azimuths)
        ele_2D_rad = np.deg2rad(elevations)
    else:
        azi_2D_rad = np.broadcast_to(
            np.deg2rad(radar.azimuth['data'])[:, np.newaxis],
            (radar.nrays, radar.ngates))
        ele_2D_rad = np.broadcast_to(
            np.deg2rad(radar.elevation['data'])[:, np.newaxis],
            (radar.nrays, radar.ngates))

    r_speed = pyart.config.get_metadata('velocity')

    # assuming no vertical velocity
    # r_speed['data'] = h_speed*np.cos(h_dir_rad-azi_2D_rad)*np.cos(ele_2D_rad)

    # with vertical velocity included
    r_speed['data'] = (
        (speed_h_u*np.sin(azi_2D_rad)+speed_h_v*np.cos(azi_2D_rad)) *
        np.cos(ele_2D_rad)+np.sin(ele_2D_rad)*v_speed)

    # prepare for exit
    new_dataset = {'radar_out': deepcopy(radar)}
    new_dataset['radar_out'].fields = dict()
    new_dataset['radar_out'].add_field('velocity', r_speed)

    return new_dataset, ind_rad


def process_wind_vel(procstatus, dscfg, radar_list=None):
    """
    Estimates the horizontal or vertical component of the wind from the
    radial velocity

    Parameters
    ----------
    procstatus : int
        Processing status: 0 initializing, 1 processing volume,
        2 post-processing
    dscfg : dictionary of dictionaries
        data set configuration. Accepted Configuration Keywords::

        datatype : string. Dataset keyword
            The input data type
        vert_proj : Boolean
            If true the vertical projection is computed. Otherwise the
            horizontal projection is computed
    radar_list : list of Radar objects
        Optional. list of radar objects

    Returns
    -------
    new_dataset : dict
        dictionary containing the output
    ind_rad : int
        radar index

    """
    if procstatus != 1:
        return None, None

    radarnr, _, datatype, _, _ = get_datatype_fields(dscfg['datatype'][0])
    vel_field = get_fieldname_pyart(datatype)

    ind_rad = int(radarnr[5:8])-1
    if radar_list[ind_rad] is None:
        warn('No valid radar')
        return None, None
    radar = radar_list[ind_rad]

    if vel_field not in radar.fields:
        warn('Unable to retrieve wind speed. Missing data')
        return None, None

    vert_proj = dscfg.get('vert_proj', False)
    wind_field = 'azimuthal_horizontal_wind_component'
    if vert_proj:
        wind_field = 'vertical_wind_component'

    wind = pyart.retrieve.est_wind_vel(
        radar, vert_proj=vert_proj, vel_field=vel_field,
        wind_field=wind_field)

    # prepare for exit
    new_dataset = {'radar_out': deepcopy(radar)}
    new_dataset['radar_out'].fields = dict()
    new_dataset['radar_out'].add_field(wind_field, wind)

    return new_dataset, ind_rad


def process_windshear(procstatus, dscfg, radar_list=None):
    """
    Estimates the wind shear from the wind velocity

    Parameters
    ----------
    procstatus : int
        Processing status: 0 initializing, 1 processing volume,
        2 post-processing
    dscfg : dictionary of dictionaries
        data set configuration. Accepted Configuration Keywords::

        datatype : string. Dataset keyword
            The input data type
        az_tol : float
            The tolerance in azimuth when looking for gates on top
            of the gate when computation is performed

    radar_list : list of Radar objects
        Optional. list of radar objects

    Returns
    -------
    new_dataset : dict
        dictionary containing the output
    ind_rad : int
        radar index

    """
    if procstatus != 1:
        return None, None

    radarnr, _, datatype, _, _ = get_datatype_fields(dscfg['datatype'][0])
    wind_field = get_fieldname_pyart(datatype)

    ind_rad = int(radarnr[5:8])-1
    if radar_list[ind_rad] is None:
        warn('No valid radar')
        return None, None
    radar = radar_list[ind_rad]

    if wind_field not in radar.fields:
        warn('Unable to retrieve wind shear. Missing data')
        return None, None

    az_tol = dscfg.get('az_tol', 0.5)
    windshear_field = 'vertical_wind_shear'

    windshear = pyart.retrieve.est_vertical_windshear(
        radar, az_tol=az_tol, wind_field=wind_field,
        windshear_field=windshear_field)

    # prepare for exit
    new_dataset = {'radar_out': deepcopy(radar)}
    new_dataset['radar_out'].fields = dict()
    new_dataset['radar_out'].add_field(windshear_field, windshear)

    return new_dataset, ind_rad


def process_vad(procstatus, dscfg, radar_list=None):
    """
    Estimates vertical wind profile using the VAD (velocity Azimuth Display)
    technique

    Parameters
    ----------
    procstatus : int
        Processing status: 0 initializing, 1 processing volume,
        2 post-processing
    dscfg : dictionary of dictionaries
        data set configuration. Accepted Configuration Keywords::

        datatype : string. Dataset keyword
            The input data type
    radar_list : list of Radar objects
        Optional. list of radar objects

    Returns
    -------
    new_dataset : dict
        dictionary containing the output
    ind_rad : int
        radar index

    """
    if procstatus != 1:
        return None, None

    radarnr, _, datatype, _, _ = get_datatype_fields(dscfg['datatype'][0])
    vel_field = get_fieldname_pyart(datatype)

    ind_rad = int(radarnr[5:8])-1
    if radar_list[ind_rad] is None:
        warn('No valid radar')
        return None, None
    radar = radar_list[ind_rad]

    if vel_field not in radar.fields:
        warn('Unable to retrieve wind speed. Missing data')
        return None, None

    # User defined parameters
    npoints_min = dscfg.get('npoints_min', 6)
    azi_spacing_max = dscfg.get('azi_spacing_max', 45.)
    vel_diff_max = dscfg.get('vel_diff_max', 10.)

    (u_vel_dict, v_vel_dict, w_vel_dict, vel_est_dict, vel_std_dict,
     vel_diff_dict) = pyart.retrieve.est_wind_profile(
         radar, npoints_min=npoints_min, azi_spacing_max=azi_spacing_max,
         vel_diff_max=vel_diff_max, rad_vel_field=vel_field,
         u_vel_field='eastward_wind_component',
         v_vel_field='northward_wind_component',
         w_vel_field='vertical_wind_component',
         vel_est_field='retrieved_velocity',
         vel_std_field='retrieved_velocity_std',
         vel_diff_field='velocity_difference')

    # prepare for exit
    new_dataset = {'radar_out': deepcopy(radar)}
    new_dataset['radar_out'].fields = dict()
    new_dataset['radar_out'].add_field('eastward_wind_component', u_vel_dict)
    new_dataset['radar_out'].add_field('northward_wind_component', v_vel_dict)
    new_dataset['radar_out'].add_field('vertical_wind_component', w_vel_dict)
    new_dataset['radar_out'].add_field('retrieved_velocity', vel_est_dict)
    new_dataset['radar_out'].add_field('retrieved_velocity_std', vel_std_dict)
    new_dataset['radar_out'].add_field('velocity_difference', vel_diff_dict)

    return new_dataset, ind_rad




# THIS FILE IS GENERATED FROM PYRAD_PROC SETUP.PY
short_version = '1.3.2'
version = '1.3.2'
full_version = '1.3.2'
git_revision = '7ffe3891028cffd3eeeac89617560e9c4d4bbcad'
compile_date_time = '2022-08-16 20:04'
username = 'wolfensb'
release = True

if not release:
    version = full_version



"""
pyrad.io.read_data_dem
========================

Functions for reading data derived from Digital Elevation Models (DEM)

.. autosummary::
    :toctree: generated/

    dem2radar_data
    read_idrisi_data
    read_idrisi_metadata
    _prepare_for_interpolation


"""
import pathlib
from warnings import warn
import numpy as np
import pandas as pd
from scipy.interpolate import RegularGridInterpolator

# check existence of gdal
try:
    from osgeo import gdal, osr
    _GDAL_AVAILABLE = True
except ImportError:
    try:
        import gdal
        import osr
        _GDAL_AVAILABLE = True
    except ImportError:
        _GDAL_AVAILABLE = False

import pyart
from pyart.config import get_metadata
from ..io.read_data_cosmo import _put_radar_in_swiss_coord

# from memory_profiler import profile

# import time


def dem2radar_data(radar, dem_data, slice_xy=True, field_name='visibility'):
    """
    get the DEM value corresponding to each radar gate using nearest
    neighbour interpolation

    Parameters
    ----------
    radar : Radar
        the radar object containing the information on the position of the
        radar gates
    dem_data : dict
        dictionary containing the DEM data
    slice_xy : boolean
        if true the horizontal plane of the DEM field is cut to the
        dimensions of the radar field
    field_names : str
        names of DEM fields to convert

    Returns
    -------
    dem_field : dict
        Dictionary with the DEM fields and metadata

    """
    # debugging
    # start_time = time.time()

    x_radar, y_radar, _ = _put_radar_in_swiss_coord(radar)

    (x_dem, y_dem, ind_xmin, ind_ymin, ind_xmax, ind_ymax) = (
        _prepare_for_interpolation(
            x_radar, y_radar, dem_data, slice_xy=slice_xy))

    if field_name not in dem_data:
        warn('DEM field '+field_name+' data not available')
        return None

    values = dem_data[field_name]['data'][
        ind_xmin:ind_xmax+1, ind_ymin:ind_ymax+1]


    # Note RegularGridInterpolator is 10x faster than NDNearestInterpolator
    # and has the advantage of not extrapolating outside of grid domain

    # replace masked values with nans
    values = np.ma.filled(values, np.nan)
    interp_func = RegularGridInterpolator(
        (x_dem, y_dem), values, bounds_error = False)


    # interpolate
    data_interp = interp_func((x_radar, y_radar))

    del values
    # restore mask
    data_interp = np.ma.masked_equal(data_interp, np.nan)

    # put field
    field_dict = get_metadata(field_name)
    field_dict['data'] = data_interp.astype(float)

    del data_interp

    return field_dict

# @profile
def read_dem(fname, field_name = 'terrain_altitude', fill_value=None,
             projparams = None):
    """
    Generic reader that reads DEM data from any format, will infer the proper
    reader from filename extension

    Parameters
    ----------
    fname : str
        name of the file to read
    field_name : str
        name of the readed variable
    fill_value : float
        The fill value, if not provided will be infered from metadata
        if possible
    projparams : projection transform as can be used by gdal either a  Proj4 
        string, see epsg.io for a list, or a EPSG number, if not provided
        will be infered from the file, or for ASCII, LV1903 will be used

    Returns
    -------
    dem_data : dictionary
        dictionary with the data and metadata

    """
    extension = pathlib.Path(fname).suffix

    if isinstance(projparams, int): # Retrieve Wkt code from EPSG number
        proj = osr.SpatialReference()
        proj.ImportFromEPSG(projparams)
        projparams = proj.ExportToProj4()
    
    projparams = _proj4_str_to_dict(projparams) 
    
    if extension in ['.tif','.tiff','.gtif']:
        metadata, rasterarray =   read_geotiff_data(fname, fill_value)
    elif extension in ['.asc','.dem','.txt']:
        metadata, rasterarray =  read_ascii_data(fname, fill_value)
    elif extension in ['.rst']:
        metadata, rasterarray = read_idrisi_data(fname, fill_value)
    else:
        warn('Unable to read file %s, extension must be .tif .tiff .gtif, '+
             '.asc .dem .txt .rst'.format(fname))
        return None

    field_dict = get_metadata(field_name)
    field_dict['data'] = rasterarray[::-1, :][None,:,:]
    field_dict['units'] = metadata['value units']

    x = get_metadata('x')
    y = get_metadata('y')
    z = get_metadata('z')

    orig_lat = get_metadata('origin_latitude')
    orig_lon = get_metadata('origin_longitude')
    orig_alt = get_metadata('origin_altitude')

    x['data'] = (
        np.arange(metadata['columns'])*metadata['resolution'] +
        metadata['resolution']/2.+metadata['min. X'])

    y['data'] = (
        np.arange(metadata['rows'])*metadata['resolution'] +
        metadata['resolution']/2.+metadata['min. Y'])

    z['data'] = np.array([0])

    orig_lat['data'] = [y['data'][0]]
    orig_lon['data'] = [x['data'][0]]
    orig_alt['data'] = [0]

    if projparams is None:
        projparams = _get_lv1903_wkt()
    
    time = get_metadata('grid_time')
    time['data'] = np.array([0.0])
    time['units'] = 'seconds since 2000-01-01T00:00:00Z'

    # The use of CRS().to_dict() is required to use GridMapDisplay of Pyart
    # which expects a dict for the projection attribute of the grid
    dem_data = pyart.core.Grid(time, {field_name : field_dict},
       metadata,  orig_lat, orig_lon, orig_alt, x, y, z,
       projection = projparams)

    return dem_data

# @profile
def read_geotiff_data(fname, fill_value = None):

    """
    Reads DEM data from a generic geotiff file, unit system is expected to be
    in meters!

    Parameters
    ----------
    fname : str
        name of the file to read
    field_name : str
        name of the readed variable
    fill_value : float
        The fill value, if not provided will be infered from metadata
        (recommended)
    projparams : projection transform as can be used by pyproj, either a
        EPSG code integer (21781 for CH1903 for example), or a OGC WKT or
        Proj4 string, see epsg.io for a list,
        if not provided will be infered from the idrisi file

    Returns
    -------
    dem_data : dictionary
        dictionary with the data and metadata

    """
    if not _GDAL_AVAILABLE:
        warn("gdal is required to use read_geotiff_data but is not installed")
        return None


    # read the data
    try:
        raster = gdal.Open(fname)

        width = raster.RasterXSize
        height = raster.RasterYSize
        gt = raster.GetGeoTransform()
        minx = gt[0]
        miny = gt[3] + width*gt[4] + height*gt[5]

        metadata = {}
        metadata['resolution'] = np.abs(gt[1])
        metadata['min. X'] = minx
        metadata['min. Y'] = miny
        metadata['rows'] = raster.RasterYSize
        metadata['columns'] = raster.RasterXSize
        metadata['value units'] = 'meters'
        metadata['max. X'] = (metadata['min. X'] +
                              metadata['resolution'] * metadata['columns'])
        metadata['max. Y'] = (metadata['min. Y'] +
                              metadata['resolution'] * metadata['rows'])
        metadata['flag value'] = raster.GetRasterBand(1).GetNoDataValue()

        if not fill_value:
            fill_value = metadata['flag value']

        rasterarray = raster.ReadAsArray()
        rasterarray = np.ma.masked_equal(rasterarray, fill_value)

        return metadata, rasterarray


    except EnvironmentError as ee:
        warn(str(ee))
        warn('Unable to read file '+fname)
        return None

# @profile
def read_ascii_data(fname, fill_value = None):
    """
    Reads DEM data from an ASCII file in the Swisstopo format, the
    unit system is expected to be in meters!

    Parameters
    ----------
    fname : str
        name of the file to read
    field_name : str
        name of the readed variable
    fill_value : float
        The fill value, if not provided will be infered from metadata
        (recommended)
    projparams : projection transform as can be used by pyproj, either a
        EPSG code integer (21781 for CH1903 for example), or a OGC WKT or
        Proj4 string, see epsg.io for a list,
        if not provided CH1903 (EPSG:21781) will be used


    Returns
    -------
    dem_data : dictionary
        dictionary with the data and metadata

    """

    # read the data
    try:
        asciidata = pd.read_csv(fname, header = None)

        metadata = {}
        metadata['columns'] = int(asciidata.iloc[0][0].split(' ')[1])
        metadata['rows'] = int(asciidata.iloc[1][0].split(' ')[1])
        metadata['min. X'] = float(asciidata.iloc[2][0].split(' ')[1])
        metadata['min. Y'] = float(asciidata.iloc[3][0].split(' ')[1])
        metadata['resolution'] = float(asciidata.iloc[4][0].split(' ')[1])
        metadata['flag value'] = float(asciidata.iloc[5][0].split(' ')[1])
        metadata['max. X'] = (metadata['min. X'] +
                              metadata['resolution'] * metadata['columns'])
        metadata['max. Y'] = (metadata['min. Y'] +
                              metadata['resolution'] * metadata['rows'])
        metadata['value units'] = 'm'

        if not fill_value:
            fill_value = metadata['flag value']

        raster_array = pd.read_csv(fname, skiprows = 6, header = None,
                                  sep = ' ')
        raster_array = np.array(raster_array)
        raster_array = raster_array[np.isfinite(raster_array)]
        raster_array = np.reshape(raster_array,
                                 (metadata['rows'],metadata['columns']))
        raster_array = np.ma.masked_equal(raster_array, fill_value)

        rasterarray = pd.read_csv(fname, skiprows = 6, header = None,
                                  sep = ' ')
        rasterarray = np.array(rasterarray)
        rasterarray = rasterarray[np.isfinite(rasterarray)]
        rasterarray = np.reshape(rasterarray,
                                 (metadata['rows'],metadata['columns']))
        rasterarray = np.ma.masked_equal(rasterarray, fill_value)


        return metadata, rasterarray

    except EnvironmentError as ee:
        warn(str(ee))
        warn('Unable to read file '+fname)
        return None

# @profile
def read_idrisi_data(fname, fill_value = None):
    """
    Reads DEM data from an IDRISI .rst file

    Parameters
    ----------
    fname : str
        name of the file to read
    field_name : str
        name of the readed variable
    fill_value : float
        The fill value
    projparams : projection transform as can be used by pyproj, either a
        EPSG code integer (21781 for CH1903 for example), or a OGC WKT or
        Proj4 string, see epsg.io for a list,
        if not provided will be infered from the idrisi file


    Returns
    -------
    dem_data : dictionary
        dictionary with the data and metadata

    """
    if not _GDAL_AVAILABLE:
        warn("gdal is required to use read_idrisi_data but is not installed")
        return None


    # read the data
    try:
        if fill_value is None:
            fill_value = -99.

        raster = gdal.Open(fname)
        rasterarray = raster.ReadAsArray()
        rasterarray = np.ma.masked_equal(rasterarray, fill_value)

        metadata = read_idrisi_metadata(fname)

        if metadata is None:
            return None

        return metadata, rasterarray

    except EnvironmentError as ee:
        warn(str(ee))
        warn('Unable to read file '+fname)
        return None


def read_idrisi_metadata(fname):
    """
    Reads DEM metadata from a IDRISI .rdc file

    Parameters
    ----------
    fname : str
        name of the file to read

    Returns
    -------
    metadata : dictionary
        dictionary with the metadata

    """
    # read the data
    fname_rdc = fname.replace('.rst', '.rdc')

    try:
        metadata = dict()
        with open(fname_rdc, 'r', newline='') as txtfile:
            for line in txtfile:
                strs = line.split(':')
                metadata.update({
                    strs[0].strip(): pyart.aux_io.convert_data(strs[1].strip())})

        return metadata
    except EnvironmentError:
        warn('Unable to read file '+fname_rdc)
        return None


def _prepare_for_interpolation(x_radar, y_radar, dem_coord, slice_xy=True):
    """
    prepares the DEM 2D volume for interpolation:
        1. if set slices the DEM data to the area
    covered by the radar
        2. creates the x, y grid for the interpolation

    Parameters
    ----------
    x_radar, y_radar : arrays
        The Swiss coordinates of the radar
    dem_coord : dict
        dictionary containing the DEM coordinates
    slice_xy : boolean
        if true the horizontal plane of the DEM field is cut to the
        dimensions of the radar field

    Returns
    -------
    x_dem, y_dem : 1D arrays
        arrays containing the flatten swiss coordinates of the DEM data in
        the area of interest
    ind_xmin, ind_ymin, ind_xmax, ind_ymax : ints
        the minimum and maximum indices of each dimension

    """
    nx_dem = len(dem_coord['x']['data'])
    ny_dem = len(dem_coord['y']['data'])

    if slice_xy:
        # get the D data within the radar range
        xmin = np.min(x_radar)
        xmax = np.max(x_radar)
        ymin = np.min(y_radar)
        ymax = np.max(y_radar)

        ind_xmin = np.where(dem_coord['x']['data'] < xmin)[0]
        if ind_xmin.size == 0:
            ind_xmin = 0
        else:
            ind_xmin = ind_xmin[-1]

        ind_xmax = np.where(dem_coord['x']['data'] > xmax)[0]
        if ind_xmax.size == 0:
            ind_xmax = nx_dem-1
        else:
            ind_xmax = ind_xmax[0]

        ind_ymin = np.where(dem_coord['y']['data'] < ymin)[0]
        if ind_ymin.size == 0:
            ind_ymin = 0
        else:
            ind_ymin = ind_ymin[-1]

        ind_ymax = np.where(dem_coord['y']['data'] > ymax)[0]
        if ind_ymax.size == 0:
            ind_ymax = ny_dem-1
        else:
            ind_ymax = ind_ymax[0]
    else:
        ind_xmin = 0
        ind_xmax = nx_dem-1
        ind_ymin = 0
        ind_ymax = ny_dem-1


    x_dem = dem_coord['x']['data'][ind_xmin:ind_xmax+1]
    y_dem = dem_coord['y']['data'][ind_ymin:ind_ymax+1]

    # Not used with RegularGridInterpolator
    # nx = ind_xmax-ind_xmin+1
    # ny = ind_ymax-ind_ymin+1
    # x_dem = (
    #     np.broadcast_to(x_dem.reshape(nx, 1), (nx, ny))).flatten()
    # y_dem = (
    #     np.broadcast_to(y_dem.reshape(1, ny), (nx, ny))).flatten()

    return (x_dem, y_dem, ind_xmin, ind_ymin, ind_xmax, ind_ymax)

def _proj4_str_to_dict(proj4str):
    # COnverts proj4 string to dict as can be used by part
    return dict(item.split("=") for item in proj4str.strip(' ').split("+")
            if len(item.split('=')) == 2) 
    
def _get_lv1903_proj4():
    # Gets proj4 dict for Swiss LV1903 coordinates
    if _GDAL_AVAILABLE:
        lv1903 = osr.SpatialReference( )
        lv1903.ImportFromEPSG(21781)
        lv1903 = lv1903.ExportToProj4()
        # Convert proj4 string to dict
        lv1903 = _proj4_str_to_dict(lv1903)
        if 'no_defs' not in lv1903.keys():
            lv1903['no_defs'] = 0
    else:
        # Hardcoded version
        lv1903 = {'proj': 'somerc',
                 'lat_0': 46.9524055555556,
                 'lon_0': 7.43958333333333,
                 'k_0': 1,
                 'x_0': 600000,
                 'y_0': 200000,
                 'ellps': 'bessel',
                 'units': 'm',
                 'no_defs': 0,
                 'type': 'crs'}
        
    return lv1903



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
================================================
main_precipitation_comparison
================================================

This program compares radar data with a point measurement sensor.

"""

# Author: fvj
# License: BSD 3 clause

import datetime
import atexit
import glob
from warnings import warn
import os

import numpy as np

from pyrad.io import read_ts_cum
from pyrad.graph import plot_scatter_comp
from pyrad.util import compute_1d_stats

print(__doc__)


def main():
    """
    """
    param_vec = ['RR_Z', 'RR_hydro']
    smn_station_vec = ['CIM', 'MAG', 'OTL']
    tstart = '20180401'
    tend = '20180430'

    np_radar_min = 6
    np_sensor_min = 6
    min_val = 0.2

    fbase = '/data/pyrad_products/mals_loc_dataquality/'
    img_ext = 'png'
    avg_time = 3600

    print("====== precipitation comparison started: %s" %
          datetime.datetime.utcnow().strftime("%Y-%m-%d %H:%M:%S"))
    atexit.register(_print_end_msg,
                    "====== comparison finished: ")

    startdate = datetime.datetime.strptime(tstart, '%Y%m%d')
    enddate = datetime.datetime.strptime(tend, '%Y%m%d')

    ndays = (enddate - startdate).days + 1
    print('Number of days to process: '+str(ndays)+'\n\n')

    for param in param_vec:
        ts_vec = np.array([])
        val_radar = np.ma.array([])
        np_radar = np.array([])
        val_sensor = np.ma.array([])
        np_sensor = np.array([])

        for station in smn_station_vec:
            for day in range(ndays):
                current_date = startdate + datetime.timedelta(days=day)
                day_dir = current_date.strftime("%Y-%m-%d")
                daybase = current_date.strftime("%Y%m%d")

                fpath = (fbase+day_dir+'/rg'+station+'_'+param+'/RRcum' +
                         str(avg_time)+'s/')
                fname = glob.glob(
                    fpath+daybase+'_'+str(avg_time) +
                    's_acc_ts_comp_POINT_MEASUREMENT_*.csv')
                if not fname:
                    warn('No file found in '+fpath)
                    continue
                else:
                    (ts_aux, np_radar_aux, radar_value_aux, np_sensor_aux,
                     sensor_value_aux) = read_ts_cum(fname[0])
                    ts_vec = np.append(ts_vec, ts_aux)
                    val_radar = np.ma.append(val_radar, radar_value_aux)
                    np_radar = np.append(np_radar, np_radar_aux)
                    val_sensor = np.ma.append(val_sensor, sensor_value_aux)
                    np_sensor = np.append(np_sensor, np_sensor_aux)

        # filter out undesired data
        ind = np.where(np.logical_and(
            np.logical_and(
                np_radar >= np_radar_min, np_sensor >= np_sensor_min),
            np.logical_and(val_sensor >= min_val, val_radar >= min_val)))[0]

        val_sensor = val_sensor[ind]
        val_radar = val_radar[ind]

        # compute statistics
        stats = compute_1d_stats(val_sensor, val_radar)

        # create output image
        fpath = fbase+'RR/'
        if os.path.isdir(fpath):
            pass
        else:
            os.makedirs(fpath)

        figfname = [
            startdate.strftime('%Y%m%d')+'-'+enddate.strftime('%Y%m%d')+'_' +
            str(avg_time)+'s_acc_ts_comp_'+param+'.'+img_ext]

        for i in range(len(figfname)):
            figfname[i] = fpath+figfname[i]

        labelx = 'RG (mm)'
        labely = 'Radar (mm)'
        titl = (str(avg_time)+' s Acc. Comp. '+startdate.strftime('%Y%m%d') +
                '-'+enddate.strftime('%Y%m%d'))

        metadata = (
            'npoints: '+str(stats['npoints'])+'\n' +
            'NB: '+'{:.2f}'.format(float(stats['NB']))+'\n' +
            'corr: '+'{:.2f}'.format(float(stats['corr']))+'\n' +
            'RMS: '+'{:.2f}'.format(float(stats['RMS']))+'\n' +
            'Nash: '+'{:.2f}'.format(float(stats['Nash']))+'\n')

        plot_scatter_comp(
            val_sensor, val_radar, figfname, labelx=labelx,
            labely=labely, titl=titl, axis='equal', metadata=metadata,
            dpi=300)


def _print_end_msg(text):
    """
    prints end message

    Parameters
    ----------
    text : str
        the text to be printed

    Returns
    -------
    Nothing

    """
    print(text + datetime.datetime.utcnow().strftime("%Y-%m-%d %H:%M:%S"))


# ---------------------------------------------------------
# Start main:
# ---------------------------------------------------------
if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
================================================
main_process_data_trt
================================================

This program activates the Pyrad processing to obtain
TRT cells trajectories and obtains post-processing products

"""

# Author: fvj
# License: BSD 3 clause

import datetime
import argparse
import atexit
import os
import glob
from warnings import warn

import numpy as np

from pyrad.flow.flow_control import main as pyrad_main
from pyrad.io import get_fieldname_pyart, write_trt_cell_lightning
from pyrad.io import read_profile_ts, read_histogram_ts, read_quantiles_ts
from pyrad.io import read_trt_traj_data, read_thundertracking_info
from pyrad.graph import get_field_name, get_colobar_label
from pyrad.graph import _plot_time_range

from pyart.config import get_metadata

print(__doc__)


def main():
    """
    """

    # parse the arguments
    parser = argparse.ArgumentParser(
        description='Entry to Pyrad processing framework')

    # positional arguments
    parser.add_argument(
        'proc_cfgfile', type=str, help='name of main configuration file')

    # keyword arguments
    parser.add_argument(
        '--days', type=str, default=None,
        help='Dates to process. Format YYYY-MM-DD. Coma separated')

    parser.add_argument(
        '--info_file', type=str,
        default='/store/msrad/radar/thundertracking/info/thundertracking_info.csv',
        help='configuration file path')

    parser.add_argument(
        '--years', type=str,
        default=None,
        help='Years to process. If None all years in file will be processed')

    parser.add_argument(
        '--max_rank', type=float,
        default=None,
        help='Max rank to process')

    parser.add_argument(
        '--trtbase', type=str,
        default='/store/msrad/radar/trt/',
        help='name of folder containing the TRT cell data')

    parser.add_argument(
        '--postproc', type=int,
        default=1,
        help='If true the data will be post-processed')

    parser.add_argument(
        '--radarbase', type=str,
        default='/store/msrad/radar/pyrad_products/thundertracking/',
        help='name of folder containing the radar data')

    parser.add_argument(
        '--cfgpath', type=str,
        default=os.path.expanduser('~')+'/pyrad/config/processing/',
        help='configuration file path')

    parser.add_argument(
        '--datatypes', type=str,
        default='RR,hydro,KDPc,dBZc,RhoHVc,TEMP,ZDRc',
        help='Name of the polarimetric moments to process. Coma separated')

    parser.add_argument(
        '--datasets', type=str,
        default='RR,hydro,KDPc,dBZc,RhoHVc,TEMP,ZDRc',
        help='Name of the directory containing the datasets')

    parser.add_argument(
        '--hres', type=float, default=250., help='Height resolution')

    parser.add_argument(
        '--path_structure', type=int, default=2,
        help='If true the data is at the cell center')

    args = parser.parse_args()

    print("====== PYRAD TRT data processing started: %s" %
          datetime.datetime.utcnow().strftime("%Y-%m-%d %H:%M:%S"))
    atexit.register(_print_end_msg,
                    "====== PYRAD TRT data processing finished: ")

    print('config path: '+args.cfgpath)
    print('config file: '+args.proc_cfgfile)
    print('trt path: '+args.trtbase)
    print('radar data path: '+args.radarbase)

    cfgfile_proc = args.cfgpath+args.proc_cfgfile
    trajtype = 'trt'

    if args.days is not None:
        time_dir_list = args.days.split(',')
    else:
        # get the years to process
        years = None
        if args.years is not None:
            years = list(map(int, args.years.split(',')))

        _, max_rank, _, trt_time_start, trt_time_end = read_thundertracking_info(
            args.info_file)
        trt_times = np.append(trt_time_start, trt_time_end)

        trt_dates = np.array([], dtype=datetime.date)
        for trt_time in trt_times:
            trt_dates = np.append(trt_dates, trt_time.date())
        trt_dates = np.sort(np.unique(trt_dates))
        time_dir_list = []
        for rank, trt_date in zip(max_rank, trt_dates):
            if years is not None:
                if trt_date.year not in years:
                    continue

            if args.max_rank is not None:
                if rank > args.max_rank:
                    continue

            time_dir_list.append(trt_date.strftime("%Y-%m-%d"))


    if args.postproc:
        datatype_list = args.datatypes.split(',')
        dataset_list = args.datasets.split(',')

        if np.size(datatype_list) != np.size(dataset_list):
            warn(
                str(np.size(datatype_list))+' datatypes but ' +
                str(np.size(dataset_list)) +
                ' dataset directories. Their number must be equal')
            return

    # Find all TRT files in directory
    trt_list = []
    if args.days is not None:
        for time_dir in time_dir_list:
            trt_list.extend(glob.glob(
                args.trtbase+time_dir+'/TRTC_cell_plots/All/*.trt'))
            trt_list.extend(glob.glob(
                args.trtbase+time_dir+'/TRTC_cell_plots/Some/*.trt'))
    else:
        for time_dir in time_dir_list:
            trt_list.extend(glob.glob(
                args.trtbase+time_dir+'/TRTC_cell/*_tt.trt'))

    if len(trt_list) == 0:
        warn('No valid TRT files found in '+args.trtbase)
        return

    # Pyrad data processing
    trt_cell_id_list = []
    trt_file_list = []
    for fname in trt_list:
        print('processing TRT cell file '+fname)
        try:
            infostr = os.path.basename(fname).split('.')[0]
            infostr = infostr.replace('_tt', '')
            pyrad_main(
                cfgfile_proc, trajfile=fname, infostr=infostr,
                trajtype=trajtype)
            trt_cell_id_list.append(infostr)
            trt_file_list.append(fname)
        except:
            warn('Unable to process TRT cell file '+fname)

    if not args.postproc:
        return

    # plot time series and get altitude of graupel column
    if 'hydro' in datatype_list:
        cell_ID_list = np.asarray([], dtype=int)
        time_list = np.asarray([], dtype=datetime.datetime)
        lon_list = np.asarray([], dtype=float)
        lat_list = np.asarray([], dtype=float)
        area_list = np.asarray([], dtype=float)
        rank_list = np.asarray([], dtype=float)
        rm_hmin_list = np.ma.asarray([], dtype=float)
        rm_hmax_list = np.ma.asarray([], dtype=float)

    for i, trt_cell_id in enumerate(trt_cell_id_list):
        print('\n\nPost-processing cell: '+trt_cell_id)
        dt_str = trt_cell_id[0:12]
        dt_cell = datetime.datetime.strptime(dt_str, "%Y%m%d%H%M")
        time_dir = dt_cell.strftime("%Y-%m-%d")
        for j, datatype in enumerate(datatype_list):
            dataset = dataset_list[j]
            if args.path_structure == 1:
                file_base2 = args.radarbase+time_dir+'/'+dataset+'_trt_center_traj/'
            elif args.path_structure == 0:
                file_base2 = args.radarbase+time_dir+'/'+dataset+'_trt_traj/'
            elif args.path_structure == 2:
                file_base2 = args.radarbase+time_dir+'/trt_traj_tt/'

            field_name = get_fieldname_pyart(datatype)
            field_dict = get_metadata(field_name)
            titl = 'TRT cell '+trt_cell_id+'\n'+get_field_name(
                field_dict, field_name)

            # plot time-height
            if args.path_structure == 2:
                flist = glob.glob(
                    file_base2+'PROFILE_'+dataset+'/*_'+trt_cell_id+'_rhi_profile_*_' +
                    datatype+'_hres'+str(int(args.hres))+'.csv')
            else:
                flist = glob.glob(
                    file_base2+'PROFILE/*_'+trt_cell_id+'_rhi_profile_*_' +
                    datatype+'_hres'+str(int(args.hres))+'.csv')


            if not flist:
                warn('No profile files found in '+file_base2 +
                     'PROFILE/ for TRT cell ' +
                     trt_cell_id+' with resolution '+str(args.hres))
            else:
                if args.path_structure == 1:
                    labels = ['Mean', 'Min', 'Max']
                else:
                    labels = [
                        '50.0-percentile', '25.0-percentile', '75.0-percentile']
                    if datatype == 'RhoHVc':
                        labels = [
                            '80.0-percentile', '65.0-percentile',
                            '95.0-percentile']
                    elif datatype == 'hydro':
                        labels = [
                            'Mode', '2nd most common', '3rd most common',
                            '% points mode', '% points 2nd most common',
                            '% points 3rd most common']
                    elif datatype == 'entropy' or 'prop' in datatype:
                        labels = ['Mean', 'Min', 'Max']

                tbin_edges, hbin_edges, _, data_ma, start_time = (
                    read_profile_ts(flist, labels, hres=args.hres, t_res=None))

                basepath_out = os.path.dirname(flist[0])
                fname = (
                    basepath_out+'/'+trt_cell_id+'_trt_TIME_HEIGHT_' +
                    datatype+'_hres'+str(args.hres)+'.png')

                vmin = vmax = None
                if datatype == 'RhoHVc':
                    vmin = 0.95
                    vmax = 1.00

                xlabel = (
                    'time (s from '+start_time.strftime("%Y-%m-%d %H:%M:%S") +
                    ')')
                _plot_time_range(
                    tbin_edges, hbin_edges, data_ma, field_name, [fname],
                    titl=titl, xlabel=xlabel, ylabel='height (m MSL)',
                    figsize=[10, 8], vmin=vmin, vmax=vmax, dpi=72)

                print("----- plot to '%s'" % fname)

                # Get min and max altitude of graupel/hail area
                if datatype == 'hydro':
                    (traj_ID, yyyymmddHHMM, lon, lat, _, _, _, area, _, _, _,
                     RANKr, _, _, _, _, _, _, _, _, _, _, _, _, _, _, _,
                     _) = read_trt_traj_data(trt_file_list[i])

                    hmin, hmax = get_graupel_column(
                        tbin_edges, hbin_edges, data_ma, start_time,
                        yyyymmddHHMM)

                    cell_ID_list = np.append(cell_ID_list, traj_ID)
                    time_list = np.append(time_list, yyyymmddHHMM)
                    lon_list = np.append(lon_list, lon)
                    lat_list = np.append(lat_list, lat)
                    area_list = np.append(area_list, area)
                    rank_list = np.append(rank_list, RANKr)
                    rm_hmin_list = np.ma.append(rm_hmin_list, hmin)
                    rm_hmax_list = np.ma.append(rm_hmax_list, hmax)

            # plot time-hist
            if args.path_structure == 2:
                flist = glob.glob(
                    file_base2+'HISTOGRAM_'+dataset+'/*_'+trt_cell_id+'_histogram_*_' +
                    datatype+'.csv')
            else:
                flist = glob.glob(
                    file_base2+'HISTOGRAM/*_'+trt_cell_id+'_histogram_*_' +
                    datatype+'.csv')

            if not flist:
                warn('No histogram files found in '+file_base2 +
                     'HISTOGRAM/ for TRT cell '+trt_cell_id)
            else:
                tbin_edges, bin_edges, data_ma, start_time = read_histogram_ts(
                    flist, datatype, t_res=None)

                basepath_out = os.path.dirname(flist[0])
                fname = (
                    basepath_out+'/'+trt_cell_id+'_trt_HISTOGRAM_'+datatype +
                    '.png')

                data_ma[data_ma == 0.] = np.ma.masked
                xlabel = (
                    'time (s from '+start_time.strftime("%Y-%m-%d %H:%M:%S") +
                    ')')
                _plot_time_range(
                    tbin_edges, bin_edges, data_ma, 'frequency_of_occurrence',
                    [fname], titl=titl, xlabel=xlabel,
                    ylabel=get_colobar_label(field_dict, field_name),
                    vmin=0., vmax=np.max(data_ma), figsize=[10, 8], dpi=72)

                print("----- plot to '%s'" % fname)

            # plot quantiles
            flist = glob.glob(
                file_base2+'QUANTILES/*_'+trt_cell_id+'_quantiles_*_' +
                datatype+'.csv')

            if not flist:
                warn('No quantiles files found in '+file_base2 +
                     'QUANTILES/ for TRT cell '+trt_cell_id)
                continue

            tbin_edges, qbin_edges, data_ma, start_time = read_quantiles_ts(
                flist, step=5., qmin=0., qmax=100., t_res=None)

            basepath_out = os.path.dirname(flist[0])
            fname = (
                basepath_out+'/'+trt_cell_id+'_trt_QUANTILES_'+datatype +
                '.png')

            vmin = vmax = None
            if datatype == 'RhoHVc':
                vmin = 0.95
                vmax = 1.00
            xlabel = (
                'time (s from '+start_time.strftime("%Y-%m-%d %H:%M:%S") +
                ')')
            _plot_time_range(
                tbin_edges, qbin_edges, data_ma, field_name, [fname],
                titl=titl, xlabel=xlabel, ylabel='Quantile', vmin=vmin,
                vmax=vmax, figsize=[10, 8], dpi=72)

            print("----- plot to '%s'" % fname)

    if 'hydro' in datatype_list:
        fname = args.trtbase+'cell_rimed_particles_column.csv'
        write_trt_cell_lightning(
            cell_ID_list, time_list, lon_list, lat_list, area_list,
            rank_list, rm_hmin_list, rm_hmax_list, fname)

        print("----- written to '%s'" % fname)


def get_graupel_column(tbin_edges, hbin_edges, data_ma, start_time,
                       yyyymmddHHMM):
    """
    Gets the minimum and maximum heigth of the graupel column

    Parameters
    ----------
    tbin_edges : 1D array of floats
        The time bin edges [s]
    hbin_edges : 1D array of floats
        The height bin edges [m MSL]
    data_ma : 2D array of ints
        Matrix containing the time-height hydrometeor classification
        information
    start_time : datetime object
        start time of the radar data
    yyyymmddHHMM : 1D array of datetime objects
        time steps of the TRT cell

    Returns
    -------
    hmin, hmax : 1D float arrays
        the minimum and maximum altitude of the rimed particles column

    """
    tbin_rights = tbin_edges[1:]
    hbin_lefts = hbin_edges[:-1]
    hbin_rights = hbin_edges[1:]
    H_lefts, _ = np.meshgrid(hbin_lefts, tbin_rights)
    H_lefts = np.ma.asarray(H_lefts)
    H_rights, _ = np.meshgrid(hbin_rights, tbin_rights)
    H_rights = np.ma.asarray(H_rights)
    mask = np.logical_not(np.logical_or(data_ma == 4, data_ma == 9))
    H_lefts[mask] = np.ma.masked
    H_rights[mask] = np.ma.masked
    hmin = np.ma.min(H_lefts, axis=1)
    hmax = np.ma.max(H_rights, axis=1)

    # All TRT cell time steps have a corresponding rimed column height value
    # Return the values
    if tbin_rights.size == yyyymmddHHMM.size:
        return hmin, hmax

    # Missing rimed height values. Determine those missing and put the data
    # in the right time step
    hmin_aux = np.ma.masked_all(yyyymmddHHMM.size, dtype=float)
    hmax_aux = np.ma.masked_all(yyyymmddHHMM.size, dtype=float)
    for k, dt_trt in enumerate(yyyymmddHHMM):
        for l, dt_rad in enumerate(tbin_rights):
            if dt_trt == start_time + datetime.timedelta(seconds=dt_rad):
                hmin_aux[k] = hmin[l]
                hmax_aux[k] = hmax[l]
                break
    return hmin_aux, hmax_aux


def _print_end_msg(text):
    """
    prints end message

    Parameters
    ----------
    text : str
        the text to be printed

    Returns
    -------
    Nothing

    """
    print(text + datetime.datetime.utcnow().strftime("%Y-%m-%d %H:%M:%S"))


# ---------------------------------------------------------
# Start main:
# ---------------------------------------------------------
if __name__ == "__main__":
    main()
